@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/usr/bin/env bash
 
 set -e
 
@@ -11,26 +11,27 @@ while getopts ":b:v:" opt; do
 			;;
 		v)
 			VERSION=${OPTARG}
+			echo "Version: ${VERSION}"
 			;;
 		\?)
 			echo "Invalid option: -${OPTARG}"
-			exit -1
+			exit 1
 			;;
 		:)
 			echo "Option -${OPTARG} requires an argument"
-			exit -1
+			exit 1
 			;;
 	esac
 done
 
 if [ -z "${BUILDDIR}" ]; then
-	BUILDDIR=${SCRIPT_DIR}/build
+	BUILDDIR=${SCRIPT_DIR}/builddir
 	echo "Using default build directory: ${BUILDDIR}"
 fi
 
-if [ ! -d ${BUILDDIR} ]; then
+if [ ! -d "${BUILDDIR}" ]; then
 	echo "Invalid build directory: ${BUILDDIR}"
-	exit -1
+	exit 1
 fi
 
 # the config file read by the daemon
@@ -40,11 +41,21 @@ export SPA_PLUGIN_DIR="${BUILDDIR}/spa/plugins"
 # the directory with pipewire modules
 export PIPEWIRE_MODULE_DIR="${BUILDDIR}/src/modules"
 export PATH="${BUILDDIR}/src/daemon:${BUILDDIR}/src/tools:${BUILDDIR}/src/examples:${PATH}"
-export LD_LIBRARY_PATH="${BUILDDIR}/pipewire-pulseaudio/src/:${BUILDDIR}/src/pipewire/:${BUILDDIR}/pipewire-jack/src/${LD_LIBRARY_PATH+":$LD_LIBRARY_PATH"}"
+export LD_LIBRARY_PATH="${BUILDDIR}/src/pipewire/:${BUILDDIR}/pipewire-jack/src/${LD_LIBRARY_PATH+":$LD_LIBRARY_PATH"}"
 export GST_PLUGIN_PATH="${BUILDDIR}/src/gst/${GST_PLUGIN_PATH+":${GST_PLUGIN_PATH}"}"
 # the directory with card profiles and paths
 export ACP_PATHS_DIR="${SCRIPT_DIR}/spa/plugins/alsa/mixer/paths"
 export ACP_PROFILES_DIR="${SCRIPT_DIR}/spa/plugins/alsa/mixer/profile-sets"
+# ALSA plugin directory
+export ALSA_PLUGIN_DIR="${BUILDDIR}/pipewire-alsa/alsa-plugins"
 
-# FIXME: find a nice, shell-neutral way to specify a prompt
-${SHELL}
+export PW_UNINSTALLED=1
+export PKG_CONFIG_PATH="${BUILDDIR}/meson-uninstalled/:${PKG_CONFIG_PATH}"
+
+if [ -d "${BUILDDIR}/subprojects/wireplumber" ]; then
+	# FIXME: find a nice, shell-neutral way to specify a prompt
+	"${SCRIPT_DIR}"/subprojects/wireplumber/wp-uninstalled.sh -b"${BUILDDIR}"/subprojects/wireplumber "${SHELL}"
+else
+	# FIXME: find a nice, shell-neutral way to specify a prompt
+	${SHELL}
+fi
