@@ -31,6 +31,7 @@
 #include <spa/support/cpu.h>
 #include <spa/utils/list.h>
 #include <spa/utils/names.h>
+#include <spa/utils/string.h>
 #include <spa/node/node.h>
 #include <spa/node/utils.h>
 #include <spa/node/io.h>
@@ -175,22 +176,24 @@ static int impl_node_send_command(void *object, const struct spa_command *comman
 
 static void emit_node_info(struct impl *this, bool full)
 {
+	uint64_t old = full ? this->info.change_mask : 0;
 	if (full)
 		this->info.change_mask = this->info_all;
 	if (this->info.change_mask) {
 		spa_node_emit_info(&this->hooks, &this->info);
-		this->info.change_mask = 0;
+		this->info.change_mask = old;
 	}
 }
 
 static void emit_port_info(struct impl *this, struct port *port, bool full)
 {
+	uint64_t old = full ? port->info.change_mask : 0;
 	if (full)
 		port->info.change_mask = port->info_all;
 	if (port->info.change_mask) {
 		spa_node_emit_port_info(&this->hooks,
 				port->direction, port->id, &port->info);
-		port->info.change_mask = 0;
+		port->info.change_mask = old;
 	}
 }
 
@@ -263,7 +266,8 @@ static int impl_node_add_port(void *object, enum spa_direction direction, uint32
 		this->last_port = port_id + 1;
 	port->valid = true;
 
-	spa_log_debug(this->log, NAME " %p: add port %d %d", this, port_id, this->last_port);
+	spa_log_debug(this->log, NAME " %p: add port %d:%d %d", this,
+			direction, port_id, this->last_port);
 	emit_port_info(this, port, true);
 
 	return 0;
@@ -288,7 +292,7 @@ impl_node_remove_port(void *object, enum spa_direction direction, uint32_t port_
 	}
 	spa_memzero(port, sizeof(struct port));
 
-	if (port_id == this->last_port - 1) {
+	if (port_id + 1 == this->last_port) {
 		int i;
 
 		for (i = this->last_port - 1; i >= 0; i--)
@@ -297,7 +301,8 @@ impl_node_remove_port(void *object, enum spa_direction direction, uint32_t port_
 
 		this->last_port = i + 1;
 	}
-	spa_log_debug(this->log, NAME " %p: remove port %d %d", this, port_id, this->last_port);
+	spa_log_debug(this->log, NAME " %p: remove port %d:%d %d", this,
+			direction, port_id, this->last_port);
 
 	spa_node_emit_port_info(&this->hooks, direction, port_id, NULL);
 
@@ -566,13 +571,14 @@ impl_node_port_use_buffers(void *object,
 	uint32_t i;
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
+
+	spa_log_debug(this->log, NAME " %p: use %d buffers on port %d:%d",
+			this, n_buffers, direction, port_id);
+
 	spa_return_val_if_fail(CHECK_PORT(this, direction, port_id), -EINVAL);
 
 	port = GET_PORT(this, direction, port_id);
 
-	spa_log_debug(this->log, NAME " %p: use buffers %d on port %d:%d",
-			this, n_buffers, direction, port_id);
-
 	spa_return_val_if_fail(port->have_format, -EIO);
 
 	clear_buffers(this, port);
@@ -597,8 +603,8 @@ impl_node_port_use_buffers(void *object,
 		if (direction == SPA_DIRECTION_OUTPUT)
 			queue_buffer(this, port, b);
 
-		spa_log_debug(this->log, NAME " %p: port %d.%d buffer %d n_data:%d data:%p maxsize:%d",
-				this, i, direction, port_id,
+		spa_log_debug(this->log, NAME " %p: port %d:%d buffer:%d n_data:%d data:%p maxsize:%d",
+				this, direction, port_id, i,
 				buffers[i]->n_datas, d[0].data, d[0].maxsize);
 	}
 	port->n_buffers = n_buffers;
@@ -615,13 +621,14 @@ impl_node_port_set_io(void *object,
 	struct port *port;
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
-	spa_return_val_if_fail(CHECK_PORT(this, direction, port_id), -EINVAL);
-
-	port = GET_PORT(this, direction, port_id);
 
 	spa_log_debug(this->log, NAME " %p: port %d:%d io %d %p/%zd", this,
 			direction, port_id, id, data, size);
 
+	spa_return_val_if_fail(CHECK_PORT(this, direction, port_id), -EINVAL);
+
+	port = GET_PORT(this, direction, port_id);
+
 	switch (id) {
 	case SPA_IO_Buffers:
 		port->io = data;
@@ -666,11 +673,11 @@ static int impl_node_process(void *object)
 	spa_log_trace_fp(this->log, NAME " %p: status %p %d %d",
 			this, outio, outio->status, outio->buffer_id);
 
-	if (outio->status == SPA_STATUS_HAVE_DATA)
+	if (SPA_UNLIKELY(outio->status == SPA_STATUS_HAVE_DATA))
 		return outio->status;
 
 	/* recycle */
-	if (outio->buffer_id < outport->n_buffers) {
+	if (SPA_LIKELY(outio->buffer_id < outport->n_buffers)) {
 		queue_buffer(this, outport, &outport->buffers[outio->buffer_id]);
 		outio->buffer_id = SPA_ID_INVALID;
 	}
@@ -686,10 +693,10 @@ static int impl_node_process(void *object)
 		struct spa_io_buffers *inio = NULL;
 		struct buffer *inb;
 
-		if (!inport->valid ||
+		if (SPA_UNLIKELY(!inport->valid ||
 		    (inio = inport->io) == NULL ||
 		    inio->buffer_id >= inport->n_buffers ||
-		    inio->status != SPA_STATUS_HAVE_DATA) {
+		    inio->status != SPA_STATUS_HAVE_DATA)) {
 			spa_log_trace_fp(this->log, NAME " %p: skip input idx:%d valid:%d "
 					"io:%p status:%d buf_id:%d n_buffers:%d", this,
 				i, inport->valid, inio,
@@ -711,7 +718,7 @@ static int impl_node_process(void *object)
 	}
 
 	outb = dequeue_buffer(this, outport);
-        if (outb == NULL) {
+        if (SPA_UNLIKELY(outb == NULL)) {
                 spa_log_trace(this->log, NAME " %p: out of buffers", this);
                 return -EPIPE;
         }
@@ -766,7 +773,7 @@ static int impl_get_interface(struct spa_handle *handle, const char *type, void
 
 	this = (struct impl *) handle;
 
-	if (strcmp(type, SPA_TYPE_INTERFACE_Node) == 0)
+	if (spa_streq(type, SPA_TYPE_INTERFACE_Node))
 		*interface = &this->node;
 	else
 		return -ENOENT;
