@@ -29,6 +29,8 @@
 extern "C" {
 #endif
 
+#include <math.h>
+
 #include <spa/support/dbus.h>
 #include <spa/support/log.h>
 #include <spa/support/loop.h>
@@ -58,8 +60,6 @@ extern "C" {
 
 #define PIPEWIRE_BATTERY_PROVIDER "/org/freedesktop/pipewire/battery"
 
-#define SPA_BT_HFP_HF_IPHONEACCEV_KEY_BATTERY	1
-
 #define MIN_LATENCY	512
 #define MAX_LATENCY	1024
 
@@ -152,6 +152,12 @@ extern "C" {
 
 #define HSP_HS_DEFAULT_CHANNEL  3
 
+#define SOURCE_ID_BLUETOOTH	0x1	/* Bluetooth SIG */
+#define SOURCE_ID_USB		0x2	/* USB Implementer's Forum */
+
+#define BUS_TYPE_USB		1
+#define BUS_TYPE_OTHER		255
+
 #define HFP_AUDIO_CODEC_CVSD    0x01
 #define HFP_AUDIO_CODEC_MSBC    0x02
 
@@ -161,10 +167,13 @@ extern "C" {
 
 #define SPA_BT_UNKNOWN_DELAY			0
 
+#define SPA_BT_NO_BATTERY			((uint8_t)255)
+
 /* HFP uses SBC encoding with precisely defined parameters. Hence, the size
  * of the input (number of PCM samples) and output is known up front. */
 #define MSBC_DECODED_SIZE       240
 #define MSBC_ENCODED_SIZE       60  /* 2 bytes header + 57 mSBC payload + 1 byte padding */
+#define MSBC_PAYLOAD_SIZE       57
 
 enum spa_bt_profile {
 	SPA_BT_PROFILE_NULL =		0,
@@ -200,6 +209,11 @@ static inline enum spa_bt_profile spa_bt_profile_from_uuid(const char *uuid)
 	else
 		return 0;
 }
+int spa_bt_profiles_from_json_array(const char *str);
+
+int spa_bt_format_vendor_product_id(uint16_t source_id, uint16_t vendor_id,
+		uint16_t product_id, char *vendor_str, int vendor_str_size,
+		char *product_str, int product_str_size);
 
 enum spa_bt_hfp_ag_feature {
 	SPA_BT_HFP_AG_FEATURE_NONE =			(0),
@@ -241,6 +255,29 @@ enum spa_bt_hfp_hf_feature {
 	SPA_BT_HFP_HF_FEATURE_ESCO_S4 =			(1 << 9),
 };
 
+/* https://btprodspecificationrefs.blob.core.windows.net/assigned-numbers/Assigned%20Number%20Types/Hands-Free%20Profile.pdf */
+enum spa_bt_hfp_hf_indicator {
+	SPA_BT_HFP_HF_INDICATOR_ENHANCED_SAFETY =        1,
+	SPA_BT_HFP_HF_INDICATOR_BATTERY_LEVEL =          2,
+};
+
+/* HFP Command AT+IPHONEACCEV
+ *  https://developer.apple.com/accessories/Accessory-Design-Guidelines.pdf */
+enum spa_bt_hfp_hf_iphoneaccev_keys {
+	SPA_BT_HFP_HF_IPHONEACCEV_KEY_BATTERY_LEVEL =    1,
+	SPA_BT_HFP_HF_IPHONEACCEV_KEY_DOCK_STATE =       2,
+};
+
+/* HFP Command AT+XAPL
+ *  https://developer.apple.com/accessories/Accessory-Design-Guidelines.pdf
+ * Bits 0, 5 and above are reserved. */
+enum spa_bt_hfp_hf_xapl_features {
+	SPA_BT_HFP_HF_XAPL_FEATURE_BATTERY_REPORTING =         (1 << 1),
+	SPA_BT_HFP_HF_XAPL_FEATURE_DOCKED_OR_POWERED =         (1 << 2),
+	SPA_BT_HFP_HF_XAPL_FEATURE_SIRI_STATUS_REPORTING =     (1 << 3),
+	SPA_BT_HFP_HF_XAPL_FEATURE_NOISE_REDUCTION_REPORTING = (1 << 4),
+};
+
 enum spa_bt_hfp_sdp_hf_features {
 	SPA_BT_HFP_SDP_HF_FEATURE_NONE =		(0),
 	SPA_BT_HFP_SDP_HF_FEATURE_ECNR =		(1 << 0),
@@ -285,6 +322,11 @@ struct spa_bt_adapter {
 	char *alias;
 	char *address;
 	char *name;
+	int bus_type;
+	uint16_t source_id;
+	uint16_t vendor_id;
+	uint16_t product_id;
+	uint16_t version_id;
 	uint32_t bluetooth_class;
 	uint32_t profiles;
 	int powered;
@@ -383,6 +425,9 @@ struct spa_bt_device_events {
 
 	/** Profile configuration changed */
 	void (*profiles_changed) (void *data, uint32_t prev_profiles, uint32_t prev_connected);
+
+	/** Device freed */
+	void (*destroy) (void *data);
 };
 
 struct spa_bt_device {
@@ -397,6 +442,10 @@ struct spa_bt_device {
 	char *battery_path;
 	char *name;
 	char *icon;
+	uint16_t source_id;
+	uint16_t vendor_id;
+	uint16_t product_id;
+	uint16_t version_id;
 	uint32_t bluetooth_class;
 	uint16_t appearance;
 	uint16_t RSSI;
@@ -406,6 +455,8 @@ struct spa_bt_device {
 	int blocked;
 	uint32_t profiles;
 	uint32_t connected_profiles;
+	uint32_t reconnect_profiles;
+	int reconnect_state;
 	struct spa_source timer;
 	struct spa_list remote_endpoint_list;
 	struct spa_list transport_list;
@@ -413,6 +464,12 @@ struct spa_bt_device {
 	uint8_t battery;
 	int has_battery;
 
+	uint32_t hw_volume_profiles;
+	/* Even though A2DP volume is exposed on transport interface, the
+	 * volume activation info would not be variate between transports
+	 * under same device. So it's safe to cache activation info here. */
+	bool a2dp_volume_active[2];
+
 	struct spa_hook_list listener_list;
 	bool added;
 
@@ -425,11 +482,14 @@ struct a2dp_codec;
 
 struct spa_bt_device *spa_bt_device_find(struct spa_bt_monitor *monitor, const char *path);
 struct spa_bt_device *spa_bt_device_find_by_address(struct spa_bt_monitor *monitor, const char *remote_address, const char *local_address);
+int spa_bt_device_add_profile(struct spa_bt_device *device, enum spa_bt_profile profile);
 int spa_bt_device_connect_profile(struct spa_bt_device *device, enum spa_bt_profile profile);
 int spa_bt_device_check_profiles(struct spa_bt_device *device, bool force);
-int spa_bt_device_ensure_a2dp_codec(struct spa_bt_device *device, const struct a2dp_codec **codecs);
+int spa_bt_device_ensure_a2dp_codec(struct spa_bt_device *device, const struct a2dp_codec * const *codecs);
 bool spa_bt_device_supports_a2dp_codec(struct spa_bt_device *device, const struct a2dp_codec *codec);
 const struct a2dp_codec **spa_bt_device_get_supported_a2dp_codecs(struct spa_bt_device *device, size_t *count);
+int spa_bt_device_ensure_hfp_codec(struct spa_bt_device *device, unsigned int codec);
+int spa_bt_device_supports_hfp_codec(struct spa_bt_device *device, unsigned int codec);
 int spa_bt_device_release_transports(struct spa_bt_device *device);
 int spa_bt_device_report_battery_level(struct spa_bt_device *device, uint8_t percentage);
 
@@ -439,17 +499,26 @@ int spa_bt_device_report_battery_level(struct spa_bt_device *device, uint8_t per
 #define spa_bt_device_emit_connected(d,...)	        spa_bt_device_emit(d, connected, 0, __VA_ARGS__)
 #define spa_bt_device_emit_codec_switched(d,...)	spa_bt_device_emit(d, codec_switched, 0, __VA_ARGS__)
 #define spa_bt_device_emit_profiles_changed(d,...)	spa_bt_device_emit(d, profiles_changed, 0, __VA_ARGS__)
+#define spa_bt_device_emit_destroy(d)			spa_bt_device_emit(d, destroy, 0)
 #define spa_bt_device_add_listener(d,listener,events,data)           \
 	spa_hook_list_append(&(d)->listener_list, listener, events, data)
 
 struct spa_bt_sco_io;
 
-struct spa_bt_sco_io *spa_bt_sco_io_create(struct spa_loop *data_loop, int fd, uint16_t write_mtu, uint16_t read_mtu);
+struct spa_bt_sco_io *spa_bt_sco_io_create(struct spa_loop *data_loop, int fd, uint16_t read_mtu, uint16_t write_mtu);
 void spa_bt_sco_io_destroy(struct spa_bt_sco_io *io);
 void spa_bt_sco_io_set_source_cb(struct spa_bt_sco_io *io, int (*source_cb)(void *userdata, uint8_t *data, int size), void *userdata);
 void spa_bt_sco_io_set_sink_cb(struct spa_bt_sco_io *io, int (*sink_cb)(void *userdata), void *userdata);
 int spa_bt_sco_io_write(struct spa_bt_sco_io *io, uint8_t *data, int size);
 
+#define SPA_BT_VOLUME_ID_RX	0
+#define SPA_BT_VOLUME_ID_TX	1
+#define SPA_BT_VOLUME_ID_TERM	2
+
+#define SPA_BT_VOLUME_INVALID	-1
+#define SPA_BT_VOLUME_HS_MAX	15
+#define SPA_BT_VOLUME_A2DP_MAX	127
+
 enum spa_bt_transport_state {
         SPA_BT_TRANSPORT_STATE_IDLE,
         SPA_BT_TRANSPORT_STATE_PENDING,
@@ -461,8 +530,10 @@ struct spa_bt_transport_events {
 	uint32_t version;
 
 	void (*destroy) (void *data);
+	void (*delay_changed) (void *data);
 	void (*state_changed) (void *data, enum spa_bt_transport_state old,
 			enum spa_bt_transport_state state);
+	void (*volume_changed) (void *data);
 };
 
 struct spa_bt_transport_implementation {
@@ -471,9 +542,20 @@ struct spa_bt_transport_implementation {
 
 	int (*acquire) (void *data, bool optional);
 	int (*release) (void *data);
+	int (*set_volume) (void *data, int id, float volume);
 	int (*destroy) (void *data);
 };
 
+struct spa_bt_transport_volume {
+	bool active;
+	float volume;
+	int hw_volume_max;
+
+	/* XXX: items below should be put to user_data */
+	int hw_volume;
+	int new_hw_volume;
+};
+
 struct spa_bt_transport {
 	struct spa_list link;
 	struct spa_bt_monitor *monitor;
@@ -491,37 +573,47 @@ struct spa_bt_transport {
 	uint32_t n_channels;
 	uint32_t channels[64];
 
+	struct spa_bt_transport_volume volumes[SPA_BT_VOLUME_ID_TERM];
+
 	int acquire_refcount;
 	int fd;
 	uint16_t read_mtu;
 	uint16_t write_mtu;
 	uint16_t delay;
-	void *user_data;
+
 	struct spa_bt_sco_io *sco_io;
 
+	struct spa_source volume_timer;
 	struct spa_source release_timer;
 
 	struct spa_hook_list listener_list;
 	struct spa_callbacks impl;
+
+	/* user_data must be the last item in the struct */
+	void *user_data;
 };
 
 struct spa_bt_transport *spa_bt_transport_create(struct spa_bt_monitor *monitor, char *path, size_t extra);
 void spa_bt_transport_free(struct spa_bt_transport *transport);
+void spa_bt_transport_set_state(struct spa_bt_transport *transport, enum spa_bt_transport_state state);
 struct spa_bt_transport *spa_bt_transport_find(struct spa_bt_monitor *monitor, const char *path);
 struct spa_bt_transport *spa_bt_transport_find_full(struct spa_bt_monitor *monitor,
                                                     bool (*callback) (struct spa_bt_transport *t, const void *data),
                                                     const void *data);
-int64_t spa_bt_transport_get_delay_nsec(struct spa_bt_transport *t);
+int64_t spa_bt_transport_get_delay_nsec(struct spa_bt_transport *transport);
+bool spa_bt_transport_volume_enabled(struct spa_bt_transport *transport);
 
 int spa_bt_transport_acquire(struct spa_bt_transport *t, bool optional);
 int spa_bt_transport_release(struct spa_bt_transport *t);
-void spa_bt_transport_ensure_sco_io(struct spa_bt_transport *t, struct spa_loop *data_loop);
+int spa_bt_transport_ensure_sco_io(struct spa_bt_transport *t, struct spa_loop *data_loop);
 
 #define spa_bt_transport_emit(t,m,v,...)		spa_hook_list_call(&(t)->listener_list, \
 								struct spa_bt_transport_events,	\
 								m, v, ##__VA_ARGS__)
 #define spa_bt_transport_emit_destroy(t)		spa_bt_transport_emit(t, destroy, 0)
+#define spa_bt_transport_emit_delay_changed(t)		spa_bt_transport_emit(t, delay_changed, 0)
 #define spa_bt_transport_emit_state_changed(t,...)	spa_bt_transport_emit(t, state_changed, 0, __VA_ARGS__)
+#define spa_bt_transport_emit_volume_changed(t)		spa_bt_transport_emit(t, volume_changed, 0)
 
 #define spa_bt_transport_add_listener(t,listener,events,data) \
         spa_hook_list_append(&(t)->listener_list, listener, events, data)
@@ -538,7 +630,8 @@ void spa_bt_transport_ensure_sco_io(struct spa_bt_transport *t, struct spa_loop
 	res;						\
 })
 
-#define spa_bt_transport_destroy(t)	spa_bt_transport_impl(t, destroy, 0)
+#define spa_bt_transport_destroy(t)		spa_bt_transport_impl(t, destroy, 0)
+#define spa_bt_transport_set_volume(t,...)	spa_bt_transport_impl(t, set_volume, 0, __VA_ARGS__)
 
 static inline enum spa_bt_transport_state spa_bt_transport_state_from_string(const char *value)
 {
@@ -552,27 +645,107 @@ static inline enum spa_bt_transport_state spa_bt_transport_state_from_string(con
 		return SPA_BT_TRANSPORT_STATE_IDLE;
 }
 
+#define DEFAULT_AG_VOLUME	1.0f
+#define DEFAULT_RX_VOLUME	1.0f
+#define DEFAULT_TX_VOLUME	0.064f /* spa_bt_volume_hw_to_linear(40, 100) */
+
+/* AVRCP/HSP volume is considered as percentage, so map it to pulseaudio (cubic) volume. */
+static inline uint32_t spa_bt_volume_linear_to_hw(double v, uint32_t hw_volume_max)
+{
+	if (v <= 0.0)
+		return 0;
+	if (v >= 1.0)
+		return hw_volume_max;
+	return SPA_CLAMP((uint64_t) lround(cbrt(v) * hw_volume_max),
+			 0u, hw_volume_max);
+}
+
+static inline double spa_bt_volume_hw_to_linear(uint32_t v, uint32_t hw_volume_max)
+{
+	double f;
+	if (v <= 0)
+		return 0.0;
+	if (v >= hw_volume_max)
+		return 1.0;
+	f = ((double) v / hw_volume_max);
+	return f * f * f;
+}
+
+enum spa_bt_feature {
+	SPA_BT_FEATURE_MSBC		= (1 << 0),
+	SPA_BT_FEATURE_MSBC_ALT1	= (1 << 1),
+	SPA_BT_FEATURE_MSBC_ALT1_RTL	= (1 << 2),
+	SPA_BT_FEATURE_HW_VOLUME	= (1 << 3),
+	SPA_BT_FEATURE_HW_VOLUME_MIC	= (1 << 4),
+	SPA_BT_FEATURE_SBC_XQ		= (1 << 5),
+};
+
+struct spa_bt_quirks;
+
+struct spa_bt_quirks *spa_bt_quirks_create(const struct spa_dict *info, struct spa_log *log);
+int spa_bt_quirks_get_features(const struct spa_bt_quirks *quirks,
+		const struct spa_bt_adapter *adapter,
+		const struct spa_bt_device *device,
+		uint32_t *features);
+void spa_bt_quirks_destroy(struct spa_bt_quirks *quirks);
+
+struct spa_bt_backend_implementation {
+#define SPA_VERSION_BT_BACKEND_IMPLEMENTATION	0
+	uint32_t version;
+
+	int (*free) (void *data);
+	int (*register_profiles) (void *data);
+	int (*unregister_profiles) (void *data);
+	int (*unregistered) (void *data);
+	int (*add_filters) (void *data);
+	int (*ensure_codec) (void *data, struct spa_bt_device *device, unsigned int codec);
+	int (*supports_codec) (void *data, struct spa_bt_device *device, unsigned int codec);
+};
+
+struct spa_bt_backend {
+	struct spa_callbacks impl;
+};
+
+#define spa_bt_backend_set_implementation(b,_impl,_data) \
+			(b)->impl = SPA_CALLBACKS_INIT(_impl, _data)
+
+#define spa_bt_backend_impl(b,m,v,...)				\
+({								\
+	int res = -ENOTSUP;					\
+	if (b)							\
+		spa_callbacks_call_res(&(b)->impl,		\
+			struct spa_bt_backend_implementation,	\
+			res, m, v, ##__VA_ARGS__);		\
+	res;							\
+})
+
+#define spa_bt_backend_free(b)			spa_bt_backend_impl(b, free, 0)
+#define spa_bt_backend_register_profiles(b)	spa_bt_backend_impl(b, register_profiles, 0)
+#define spa_bt_backend_unregister_profiles(b)	spa_bt_backend_impl(b, unregister_profiles, 0)
+#define spa_bt_backend_unregistered(b)		spa_bt_backend_impl(b, unregistered, 0)
+#define spa_bt_backend_add_filters(b)		spa_bt_backend_impl(b, add_filters, 0)
+#define spa_bt_backend_ensure_codec(b,...)	spa_bt_backend_impl(b, ensure_codec, 0, __VA_ARGS__)
+#define spa_bt_backend_supports_codec(b,...)	spa_bt_backend_impl(b, supports_codec, 0, __VA_ARGS__)
+
+static inline struct spa_bt_backend *dummy_backend_new(struct spa_bt_monitor *monitor,
+		void *dbus_connection,
+		const struct spa_dict *info,
+		const struct spa_bt_quirks *quirks,
+		const struct spa_support *support,
+		uint32_t n_support)
+{
+	return NULL;
+}
 
 #ifdef HAVE_BLUEZ_5_BACKEND_NATIVE
 struct spa_bt_backend *backend_native_new(struct spa_bt_monitor *monitor,
 		void *dbus_connection,
 		const struct spa_dict *info,
+		const struct spa_bt_quirks *quirks,
 		const struct spa_support *support,
 		uint32_t n_support);
-void backend_native_free(struct spa_bt_backend *backend);
-void backend_native_register_profiles(struct spa_bt_backend *backend);
-void backend_native_unregister_profiles(struct spa_bt_backend *backend);
 #else
-static inline struct spa_bt_backend *backend_native_new(struct spa_bt_monitor *monitor,
-		void *dbus_connection,
-		const struct spa_dict *info,
-		const struct spa_support *support,
-		uint32_t n_support) {
-	return NULL;
-}
-static inline void backend_native_free(struct spa_bt_backend *backend) {}
-static inline void backend_native_register_profiles(struct spa_bt_backend *backend) {}
-static inline void backend_native_unregister_profiles(struct spa_bt_backend *backend) {}
+#define backend_native_new	dummy_backend_new
 #endif
 
 #define OFONO_SERVICE "org.ofono"
@@ -580,22 +753,11 @@ static inline void backend_native_unregister_profiles(struct spa_bt_backend *bac
 struct spa_bt_backend *backend_ofono_new(struct spa_bt_monitor *monitor,
 		void *dbus_connection,
 		const struct spa_dict *info,
+		const struct spa_bt_quirks *quirks,
 		const struct spa_support *support,
 		uint32_t n_support);
-void backend_ofono_free(struct spa_bt_backend *backend);
-int backend_ofono_register(struct spa_bt_backend *backend);
-void backend_ofono_add_filters(struct spa_bt_backend *backend);
 #else
-static inline struct spa_bt_backend *backend_ofono_new(struct spa_bt_monitor *monitor,
-		void *dbus_connection,
-		const struct spa_dict *info,
-		const struct spa_support *support,
-		uint32_t n_support) {
-	return NULL;
-}
-static inline void backend_ofono_free(struct spa_bt_backend *backend) {}
-static inline int backend_ofono_register(struct spa_bt_backend *backend) { return -ENOTSUP; }
-static inline void backend_ofono_add_filters(struct spa_bt_backend *backend) {}
+#define backend_ofono_new	dummy_backend_new
 #endif
 
 #define HSPHFPD_SERVICE "org.hsphfpd"
@@ -603,24 +765,11 @@ static inline void backend_ofono_add_filters(struct spa_bt_backend *backend) {}
 struct spa_bt_backend *backend_hsphfpd_new(struct spa_bt_monitor *monitor,
 		void *dbus_connection,
 		const struct spa_dict *info,
+		const struct spa_bt_quirks *quirks,
 		const struct spa_support *support,
 		uint32_t n_support);
-void backend_hsphfpd_free(struct spa_bt_backend *backend);
-int backend_hsphfpd_register(struct spa_bt_backend *backend);
-void backend_hsphfpd_unregistered(struct spa_bt_backend *backend);
-void backend_hsphfpd_add_filters(struct spa_bt_backend *backend);
 #else
-static inline struct spa_bt_backend *backend_hsphfpd_new(struct spa_bt_monitor *monitor,
-		void *dbus_connection,
-		const struct spa_dict *info,
-		const struct spa_support *support,
-		uint32_t n_support) {
-	return NULL;
-}
-static inline void backend_hsphfpd_free(struct spa_bt_backend *backend) {}
-static inline int backend_hsphfpd_register(struct spa_bt_backend *backend) { return -ENOTSUP; }
-static inline void backend_hsphfpd_unregistered(struct spa_bt_backend *backend) {}
-static inline void backend_hsphfpd_add_filters(struct spa_bt_backend *backend) {}
+#define backend_hsphfpd_new	dummy_backend_new
 #endif
 
 #ifdef __cplusplus
