@@ -37,6 +37,7 @@
 #include <spa/utils/result.h>
 #include <spa/utils/names.h>
 #include <spa/utils/keys.h>
+#include <spa/utils/string.h>
 #include <spa/pod/builder.h>
 #include <spa/pod/parser.h>
 #include <spa/param/props.h>
@@ -46,8 +47,12 @@
 #include "pipewire/impl.h"
 #include "media-session.h"
 
+/** \page page_media_session_module_bluez_monitor Media Session Module: BlueZ Monitor
+ */
+
 #define NAME		"bluez5-monitor"
 #define SESSION_CONF	"bluez-monitor.conf"
+#define FEATURES_CONF	"bluez-hardware.conf"
 
 struct device;
 
@@ -121,30 +126,30 @@ static void update_icon_name(struct pw_properties *p, bool is_sink)
 	const char *s, *d = NULL, *bus;
 
 	if ((s = pw_properties_get(p, PW_KEY_DEVICE_FORM_FACTOR))) {
-		if (strcmp(s, "microphone") == 0)
+		if (spa_streq(s, "microphone"))
 			d = "audio-input-microphone";
-		else if (strcmp(s, "webcam") == 0)
+		else if (spa_streq(s, "webcam"))
 			d = "camera-web";
-		else if (strcmp(s, "computer") == 0)
+		else if (spa_streq(s, "computer"))
 			d = "computer";
-		else if (strcmp(s, "handset") == 0)
+		else if (spa_streq(s, "handset"))
 			d = "phone";
-		else if (strcmp(s, "portable") == 0)
+		else if (spa_streq(s, "portable"))
 			d = "multimedia-player";
-		else if (strcmp(s, "tv") == 0)
+		else if (spa_streq(s, "tv"))
 			d = "video-display";
-		else if (strcmp(s, "headset") == 0)
+		else if (spa_streq(s, "headset"))
 			d = "audio-headset";
-		else if (strcmp(s, "headphone") == 0)
+		else if (spa_streq(s, "headphone"))
 			d = "audio-headphones";
-		else if (strcmp(s, "speaker") == 0)
+		else if (spa_streq(s, "speaker"))
 			d = "audio-speakers";
-		else if (strcmp(s, "hands-free") == 0)
+		else if (spa_streq(s, "hands-free"))
 			d = "audio-handsfree";
 	}
 	if (!d)
 		if ((s = pw_properties_get(p, PW_KEY_DEVICE_CLASS)))
-			if (strcmp(s, "modem") == 0)
+			if (spa_streq(s, "modem"))
 				d = "modem";
 
 	if (!d) {
@@ -195,7 +200,7 @@ static struct node *bluez5_create_node(struct device *device, uint32_t id,
 
 	pw_log_debug("new node %u", id);
 
-	if (strcmp(info->type, SPA_TYPE_INTERFACE_Node) != 0) {
+	if (!spa_streq(info->type, SPA_TYPE_INTERFACE_Node)) {
 		errno = EINVAL;
 		return NULL;
 	}
@@ -403,7 +408,7 @@ static int update_device_props(struct device *device)
 		snprintf(temp, sizeof(temp), "%d", device->id);
 		s = temp;
 	}
-	if (strstr(s, "bluez_card.") == s)
+	if (spa_strstartswith(s, "bluez_card."))
 		s += strlen("bluez_card.");
 
 	pw_properties_set(p, PW_KEY_DEVICE_NAME,
@@ -471,7 +476,7 @@ static struct device *bluez5_create_device(struct impl *impl, uint32_t id,
 
 	pw_log_debug("new device %u", id);
 
-	if (strcmp(info->type, SPA_TYPE_INTERFACE_Device) != 0) {
+	if (!spa_streq(info->type, SPA_TYPE_INTERFACE_Device)) {
 		errno = EINVAL;
 		return NULL;
 	}
@@ -563,7 +568,7 @@ static void bluez5_update_device(struct impl *impl, struct device *device,
 	update_device_props(device);
 
 	str = spa_dict_lookup(info->props, SPA_KEY_API_BLUEZ5_CONNECTION);
-	connected = str != NULL && strcmp(str, "connected") == 0;
+	connected = str != NULL && spa_streq(str, "connected");
 
 	/* Export device after bluez profiles get connected */
 	if (device->sdevice == NULL && connected) {
@@ -714,7 +719,9 @@ int sm_bluez5_monitor_start(struct sm_media_session *session)
 {
 	int res;
 	struct impl *impl;
-	const char *str;
+	const char *key, *str;
+	struct pw_properties *hw_features = NULL;
+	void *state = NULL;
 
 	impl = calloc(1, sizeof(struct impl));
 	if (impl == NULL) {
@@ -730,9 +737,16 @@ int sm_bluez5_monitor_start(struct sm_media_session *session)
 		res = -errno;
 		goto out_free;
 	}
+	if ((hw_features = pw_properties_new(NULL, NULL)) == NULL) {
+		res = -errno;
+		goto out_free;
+	}
 	if ((res = sm_media_session_load_conf(impl->session,
 					SESSION_CONF, impl->conf)) < 0)
 		pw_log_info("can't load "SESSION_CONF" config: %s", spa_strerror(res));
+	if ((res = sm_media_session_load_conf(impl->session,
+					FEATURES_CONF, hw_features)) < 0)
+		pw_log_info("can't load "FEATURES_CONF" config: %s", spa_strerror(res));
 
 	if ((impl->props = pw_properties_new(NULL, NULL)) == NULL) {
 		res = -errno;
@@ -743,16 +757,23 @@ int sm_bluez5_monitor_start(struct sm_media_session *session)
 
 	pw_properties_set(impl->props, "api.bluez5.connection-info", "true");
 
+	while ((key = pw_properties_iterate(hw_features, &state)) != NULL) {
+		if (strncmp(key, "bluez5.features.", strlen("bluez5.features.")) != 0)
+			continue;
+		if ((str = pw_properties_get(hw_features, key)) != NULL)
+			pw_properties_set(impl->props, key, str);
+	}
+
+	pw_properties_free(hw_features);
+
 	sm_media_session_add_listener(session, &impl->session_listener,
 			&session_events, impl);
-
 	return 0;
 
 out_free:
-	if (impl->conf)
-		pw_properties_free(impl->conf);
-	if (impl->props)
-		pw_properties_free(impl->props);
+	pw_properties_free(impl->conf);
+	pw_properties_free(impl->props);
+	pw_properties_free(hw_features);
 	free(impl);
 out:
 	return res;
