@@ -22,23 +22,21 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <errno.h>
-#include <time.h>
+#include <signal.h>
 #include <math.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/mman.h>
 
 #include <spa/param/video/format-utils.h>
-#include <spa/param/props.h>
 
 #include <pipewire/pipewire.h>
 
-#define BPP	3
-#define WIDTH	320
-#define HEIGHT	200
-#define CROP	8
+#define BPP		3
 #define CURSOR_WIDTH	64
 #define CURSOR_HEIGHT	64
 #define CURSOR_BPP	4
@@ -48,7 +46,7 @@
 #define M_PI_M2 ( M_PI + M_PI )
 
 struct data {
-	struct pw_main_loop *loop;
+	struct pw_thread_loop *loop;
 	struct spa_source *timer;
 
 	struct pw_stream *stream;
@@ -82,8 +80,8 @@ static void draw_elipse(uint32_t *dst, int width, int height, uint32_t color)
 	}
 }
 
-/* called on timeout and we should push a new buffer in the queue */
-static void on_timeout(void *userdata, uint64_t expirations)
+/* called when we should push a new buffer in the queue */
+static void on_process(void *userdata)
 {
 	struct data *data = userdata;
 	struct pw_buffer *b;
@@ -95,8 +93,6 @@ static void on_timeout(void *userdata, uint64_t expirations)
 	struct spa_meta_region *mc;
 	struct spa_meta_cursor *mcs;
 
-	pw_log_trace("timeout");
-
 	if ((b = pw_stream_dequeue_buffer(data->stream)) == NULL) {
 		pw_log_warn("out of buffers: %m");
 		return;
@@ -133,8 +129,8 @@ static void on_timeout(void *userdata, uint64_t expirations)
 		data->crop = (sin(data->accumulator) + 1.0) * 32.0;
 		mc->region.position.x = data->crop;
 		mc->region.position.y = data->crop;
-		mc->region.size.width = WIDTH - data->crop*2;
-		mc->region.size.height = HEIGHT - data->crop*2;
+		mc->region.size.width = data->format.size.width - data->crop*2;
+		mc->region.size.height = data->format.size.height - data->crop*2;
 	}
 	if ((mcs = spa_buffer_find_meta_data(buf, SPA_META_Cursor, sizeof(*mcs)))) {
 		struct spa_meta_bitmap *mb;
@@ -147,14 +143,14 @@ static void on_timeout(void *userdata, uint64_t expirations)
 		mcs->hotspot.y = 0;
 		mcs->bitmap_offset = sizeof(struct spa_meta_cursor);
 
-		mb = SPA_MEMBER(mcs, mcs->bitmap_offset, struct spa_meta_bitmap);
+		mb = SPA_PTROFF(mcs, mcs->bitmap_offset, struct spa_meta_bitmap);
 		mb->format = SPA_VIDEO_FORMAT_ARGB;
 		mb->size.width = CURSOR_WIDTH;
 		mb->size.height = CURSOR_HEIGHT;
 		mb->stride = CURSOR_WIDTH * CURSOR_BPP;
 		mb->offset = sizeof(struct spa_meta_bitmap);
 
-		bitmap = SPA_MEMBER(mb, mb->offset, uint32_t);
+		bitmap = SPA_PTROFF(mb, mb->offset, uint32_t);
 		color = (cos(data->accumulator) + 1.0) * (1 << 23);
 		color |= 0xff000000;
 
@@ -180,6 +176,14 @@ static void on_timeout(void *userdata, uint64_t expirations)
 	pw_stream_queue_buffer(data->stream, b);
 }
 
+/* trigger the graph when we are a driver */
+static void on_timeout(void *userdata, uint64_t expirations)
+{
+	struct data *data = userdata;
+	pw_log_trace("timeout");
+	pw_stream_trigger_process(data->stream);
+}
+
 /* when the stream is STREAMING, start the timer at 40ms intervals
  * to produce and push a frame. In other states we PAUSE the timer. */
 static void on_stream_state_changed(void *_data, enum pw_stream_state old, enum pw_stream_state state,
@@ -192,7 +196,7 @@ static void on_stream_state_changed(void *_data, enum pw_stream_state old, enum
 	switch (state) {
 	case PW_STREAM_STATE_PAUSED:
 		printf("node id: %d\n", pw_stream_get_node_id(data->stream));
-		pw_loop_update_timer(pw_main_loop_get_loop(data->loop),
+		pw_loop_update_timer(pw_thread_loop_get_loop(data->loop),
 				data->timer, NULL, NULL, false);
 		break;
 	case PW_STREAM_STATE_STREAMING:
@@ -204,8 +208,9 @@ static void on_stream_state_changed(void *_data, enum pw_stream_state old, enum
 		interval.tv_sec = 0;
 		interval.tv_nsec = 40 * SPA_NSEC_PER_MSEC;
 
-		pw_loop_update_timer(pw_main_loop_get_loop(data->loop),
-				data->timer, &timeout, &interval, false);
+		if (pw_stream_is_driving(data->stream))
+			pw_loop_update_timer(pw_thread_loop_get_loop(data->loop),
+					data->timer, &timeout, &interval, false);
 		break;
 	}
 	default:
@@ -220,8 +225,11 @@ static void on_stream_add_buffer(void *_data, struct pw_buffer *buffer)
 	struct data *data = _data;
 	struct spa_buffer *buf = buffer->buffer;
 	struct spa_data *d;
+#ifdef HAVE_MEMFD_CREATE
 	unsigned int seals;
+#endif
 
+	pw_log_info("add buffer %p", buffer);
 	d = buf->datas;
 
 	if ((d[0].type & (1<<SPA_DATA_MemFd)) == 0) {
@@ -232,7 +240,7 @@ static void on_stream_add_buffer(void *_data, struct pw_buffer *buffer)
 	/* create the memfd on the buffer, set the type and flags */
 	d[0].type = SPA_DATA_MemFd;
 	d[0].flags = SPA_DATA_FLAG_READWRITE;
-#ifndef __FreeBSD__
+#ifdef HAVE_MEMFD_CREATE
 	d[0].fd = memfd_create("video-src-memfd", MFD_CLOEXEC | MFD_ALLOW_SEALING);
 #else
 	d[0].fd = -1;
@@ -249,7 +257,7 @@ static void on_stream_add_buffer(void *_data, struct pw_buffer *buffer)
 		pw_log_error("can't truncate to %d: %m", d[0].maxsize);
 		return;
 	}
-#ifndef __FreeBSD__
+#ifdef HAVE_MEMFD_CREATE
 	/* not enforced yet but server might require SEAL_SHRINK later */
 	seals = F_SEAL_GROW | F_SEAL_SHRINK | F_SEAL_SEAL;
 	if (fcntl(d[0].fd, F_ADD_SEALS, seals) == -1) {
@@ -273,6 +281,7 @@ static void on_stream_remove_buffer(void *_data, struct pw_buffer *buffer)
 	struct spa_data *d;
 
 	d = buf->datas;
+	pw_log_info("remove buffer %p", buffer);
 
 	munmap(d[0].data, d[0].maxsize);
 	close(d[0].fd);
@@ -343,12 +352,19 @@ on_stream_param_changed(void *_data, uint32_t id, const struct spa_pod *param)
 
 static const struct pw_stream_events stream_events = {
 	PW_VERSION_STREAM_EVENTS,
+	.process = on_process,
 	.state_changed = on_stream_state_changed,
 	.param_changed = on_stream_param_changed,
 	.add_buffer = on_stream_add_buffer,
 	.remove_buffer = on_stream_remove_buffer,
 };
 
+static void do_quit(void *userdata, int signal_number)
+{
+	struct data *data = userdata;
+	pw_thread_loop_signal(data->loop, false);
+}
+
 int main(int argc, char *argv[])
 {
 	struct data data = { 0, };
@@ -358,8 +374,20 @@ int main(int argc, char *argv[])
 
 	pw_init(&argc, &argv);
 
-	/* create a main loop */
-	data.loop = pw_main_loop_new(NULL);
+	/* create a thread loop and start it */
+	data.loop = pw_thread_loop_new("video-src-alloc", NULL);
+
+	/* take the lock around all PipeWire functions. In callbacks, the lock
+	 * is already taken for you but it's ok to lock again because the lock is
+	 * recursive */
+	pw_thread_loop_lock(data.loop);
+
+	/* install some handlers to exit nicely */
+	pw_loop_add_signal(pw_thread_loop_get_loop(data.loop), SIGINT, do_quit, &data);
+	pw_loop_add_signal(pw_thread_loop_get_loop(data.loop), SIGTERM, do_quit, &data);
+
+	/* start after the signal handlers are set */
+	pw_thread_loop_start(data.loop);
 
 	/* create a simple stream, the simple stream manages the core
 	 * object for you if you don't want to deal with them.
@@ -373,7 +401,7 @@ int main(int argc, char *argv[])
 	 * the data.
 	 */
 	data.stream = pw_stream_new_simple(
-			pw_main_loop_get_loop(data.loop),
+			pw_thread_loop_get_loop(data.loop),
 			"video-src-alloc",
 			pw_properties_new(
 				PW_KEY_MEDIA_CLASS, "Video/Source",
@@ -382,7 +410,7 @@ int main(int argc, char *argv[])
 			&data);
 
 	/* make a timer to schedule our frames */
-	data.timer = pw_loop_add_timer(pw_main_loop_get_loop(data.loop), on_timeout, &data);
+	data.timer = pw_loop_add_timer(pw_thread_loop_get_loop(data.loop), on_timeout, &data);
 
 	/* build the extra parameter for the connection. Here we make an
 	 * EnumFormat parameter which lists the possible formats we can provide.
@@ -409,16 +437,23 @@ int main(int argc, char *argv[])
 	 * the server.  */
 	pw_stream_connect(data.stream,
 			  PW_DIRECTION_OUTPUT,
-			  PW_ID_ANY,
+			  PW_ID_ANY,			/* link to any node */
 			  PW_STREAM_FLAG_DRIVER |
 			  PW_STREAM_FLAG_ALLOC_BUFFERS,
 			  params, 1);
 
-	/* run the loop, this will trigger the callbacks */
-	pw_main_loop_run(data.loop);
+	/* unlock, run the loop and wait, this will trigger the callbacks */
+	pw_thread_loop_wait(data.loop);
+
+	/* unlock before stop */
+	pw_thread_loop_unlock(data.loop);
+	pw_thread_loop_stop(data.loop);
 
 	pw_stream_destroy(data.stream);
-	pw_main_loop_destroy(data.loop);
+
+	/* destroy after dependent objects are destroyed */
+	pw_thread_loop_destroy(data.loop);
+	pw_deinit();
 
 	return 0;
 }
