@@ -35,6 +35,7 @@
 
 #include <dbus/dbus.h>
 
+#include <spa/utils/string.h>
 #include <spa/support/dbus.h>
 
 #include "pipewire/context.h"
@@ -44,6 +45,40 @@
 #include "pipewire/utils.h"
 #include "pipewire/private.h"
 
+/** \page page_module_portal PipeWire Module: Portal
+ *
+ * The `portal` module performs access control management for clients started
+ * inside an XDG portal.
+ *
+ * The module connects to the session DBus and subscribes to
+ * `NameOwnerChanged` signals for the `org.freedesktop.portal.Desktop` name.
+ * The PID of the DBus name owner is the portal.
+ *
+ * A client connection from the portal PID to PipeWire gets assigned a \ref
+ * PW_KEY_ACCESS of `"portal"` and set to permissions ALL - it is the
+ * responsibility of the portal to limit the permissions before passing the
+ * connection on to the client. See \ref page_access for details on
+ * permissions.
+ *
+ * Clients connecting from other PIDs are ignored by this module.
+ *
+ * ## Module Options
+ *
+ * There are no module-specific options.
+ *
+ * ## General options
+ *
+ * There are no general options for this module.
+ *
+ * ## Example configuration
+ *\code{.unparsed}
+ * context.modules = [
+ *  {   name = libpipewire-portal }
+ * ]
+ *\endcode
+ *
+ */
+
 #define NAME "portal"
 
 struct impl {
@@ -79,8 +114,7 @@ context_check_access(void *data, struct pw_impl_client *client)
 	if ((str = pw_properties_get(props, PW_KEY_SEC_PID)) == NULL)
 		return;
 
-	pid = atoi(str);
-	if (pid != impl->portal_pid)
+	if (!spa_atoi32(str, &pid, 10) || pid != impl->portal_pid)
 		return;
 
 	items[0] = SPA_DICT_ITEM_INIT(PW_KEY_ACCESS, "portal");
@@ -107,10 +141,11 @@ static void module_destroy(void *data)
 	spa_hook_remove(&impl->context_listener);
 	spa_hook_remove(&impl->module_listener);
 
+	if (impl->bus)
+		dbus_connection_unref(impl->bus);
 	spa_dbus_connection_destroy(impl->conn);
 
-	if (impl->properties)
-		pw_properties_free(impl->properties);
+	pw_properties_free(impl->properties);
 
 	free(impl);
 }
@@ -172,7 +207,7 @@ static void update_portal_pid(struct impl *impl)
 	impl->portal_pid = 0;
 
 	m = dbus_message_new_method_call("org.freedesktop.DBus",
-					 "/",
+					 "/org/freedesktop/DBus",
 					 "org.freedesktop.DBus",
 					 "GetConnectionUnixProcessID");
 
@@ -212,10 +247,10 @@ static DBusHandlerResult name_owner_changed_handler(DBusConnection *connection,
 		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 	}
 
-	if (strcmp(name, "org.freedesktop.portal.Desktop") != 0)
+	if (!spa_streq(name, "org.freedesktop.portal.Desktop"))
 		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
-	if (strcmp(new_owner, "") == 0) {
+	if (spa_streq(new_owner, "")) {
 		impl->portal_pid = 0;
 		if (impl->portal_pid_pending != NULL) {
 			dbus_pending_call_cancel(impl->portal_pid_pending);
@@ -234,6 +269,11 @@ static int init_dbus_connection(struct impl *impl)
 	DBusError error;
 
 	impl->bus = spa_dbus_connection_get(impl->conn);
+	if (impl->bus == NULL)
+		return -EIO;
+
+	/* XXX: we don't handle dbus reconnection yet, so ref the handle instead */
+	dbus_connection_ref(impl->bus);
 
 	dbus_error_init(&error);
 
@@ -247,7 +287,7 @@ static int init_dbus_connection(struct impl *impl)
 		pw_log_error("Failed to add name owner changed listener: %s",
 			     error.message);
 		dbus_error_free(&error);
-		return -1;
+		return -EIO;
 	}
 
 	dbus_connection_add_filter(impl->bus, name_owner_changed_handler,
@@ -266,6 +306,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	struct spa_dbus *dbus;
 	const struct spa_support *support;
 	uint32_t n_support;
+	int res;
 
 	support = pw_context_get_support(context, &n_support);
 
@@ -283,10 +324,12 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	impl->properties = args ? pw_properties_new_string(args) : NULL;
 
 	impl->conn = spa_dbus_get_connection(dbus, SPA_DBUS_TYPE_SESSION);
-	if (impl->conn == NULL)
+	if (impl->conn == NULL) {
+		res = -errno;
 		goto error;
+	}
 
-	if (init_dbus_connection(impl) != 0)
+	if ((res = init_dbus_connection(impl)) < 0)
 		goto error;
 
 	pw_context_add_listener(context, &impl->context_listener, &context_events, impl);
@@ -296,6 +339,6 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 
       error:
 	free(impl);
-	pw_log_error("Failed to connect to system bus");
-	return -ENOMEM;
+	pw_log_error("Failed to connect to session bus: %s", spa_strerror(res));
+	return res;
 }
