@@ -26,7 +26,11 @@
 
 #include <spa/pod/iter.h>
 #include <spa/pod/parser.h>
-#include <extensions/metadata.h>
+#include <spa/utils/result.h>
+#include <spa/utils/string.h>
+#include <pipewire/extensions/metadata.h>
+
+#define MAX_PARAMS 32
 
 #define manager_emit_sync(m) spa_hook_list_call(&m->hooks, struct pw_manager_events, sync, 0)
 #define manager_emit_added(m,o) spa_hook_list_call(&m->hooks, struct pw_manager_events, added, 0, o)
@@ -72,6 +76,8 @@ struct object {
 	struct spa_hook proxy_listener;
 	struct spa_hook object_listener;
 
+	int param_seq[MAX_PARAMS];
+
 	struct spa_list data_list;
 };
 
@@ -97,7 +103,8 @@ static uint32_t clear_params(struct spa_list *param_list, uint32_t id)
 	return count;
 }
 
-static struct pw_manager_param *add_param(struct spa_list *params, uint32_t id, const struct spa_pod *param)
+static struct pw_manager_param *add_param(struct spa_list *params,
+		int seq, int *param_seq, uint32_t id, const struct spa_pod *param)
 {
 	struct pw_manager_param *p;
 
@@ -109,13 +116,26 @@ static struct pw_manager_param *add_param(struct spa_list *params, uint32_t id,
 		id = SPA_POD_OBJECT_ID(param);
 	}
 
+	if (id >= MAX_PARAMS) {
+		pw_log_error("too big param id %d", id);
+		errno = EINVAL;
+		return NULL;
+	}
+
+	if (seq != param_seq[id]) {
+		pw_log_debug("ignoring param %d, seq:%d != current_seq:%d",
+				id, seq, param_seq[id]);
+		errno = EBUSY;
+		return NULL;
+	}
+
 	p = malloc(sizeof(*p) + (param != NULL ? SPA_POD_SIZE(param) : 0));
 	if (p == NULL)
 		return NULL;
 
 	p->id = id;
 	if (param != NULL) {
-		p->param = SPA_MEMBER(p, sizeof(*p), struct spa_pod);
+		p->param = SPA_PTROFF(p, sizeof(*p), struct spa_pod);
 		memcpy(p->param, param, SPA_POD_SIZE(param));
 	} else {
 		clear_params(params, id);
@@ -174,8 +194,9 @@ static void object_destroy(struct object *o)
 	m->this.n_objects--;
 	if (o->this.proxy)
 		pw_proxy_destroy(o->this.proxy);
-	if (o->this.props)
-		pw_properties_free(o->this.props);
+	pw_properties_free(o->this.props);
+	if (o->this.message_object_path)
+		free(o->this.message_object_path);
 	clear_params(&o->this.param_list, SPA_ID_INVALID);
 	clear_params(&o->pending_list, SPA_ID_INVALID);
 	spa_list_consume(d, &o->data_list, link) {
@@ -197,9 +218,9 @@ static void client_event_info(void *object, const struct pw_client_info *info)
 	struct object *o = object;
 	int changed = 0;
 
-        pw_log_debug("object %p: id:%d change-mask:%08"PRIx64, o, o->this.id, info->change_mask);
+	pw_log_debug("object %p: id:%d change-mask:%08"PRIx64, o, o->this.id, info->change_mask);
 
-        info = o->this.info = pw_client_info_update(o->this.info, info);
+	info = o->this.info = pw_client_info_merge(o->this.info, info, o->this.changed == 0);
 
 	if (info->change_mask & PW_CLIENT_CHANGE_MASK_PROPS)
 		changed++;
@@ -233,12 +254,12 @@ static const struct object_info client_info = {
 /* module */
 static void module_event_info(void *object, const struct pw_module_info *info)
 {
-        struct object *o = object;
+	struct object *o = object;
 	int changed = 0;
 
-        pw_log_debug("object %p: id:%d change-mask:%08"PRIx64, o, o->this.id, info->change_mask);
+	pw_log_debug("object %p: id:%d change-mask:%08"PRIx64, o, o->this.id, info->change_mask);
 
-        info = o->this.info = pw_module_info_update(o->this.info, info);
+	info = o->this.info = pw_module_info_merge(o->this.info, info, o->this.changed == 0);
 
 	if (info->change_mask & PW_MODULE_CHANGE_MASK_PROPS)
 		changed++;
@@ -277,7 +298,7 @@ static void device_event_info(void *object, const struct pw_device_info *info)
 
 	pw_log_debug("object %p: id:%d change-mask:%08"PRIx64, o, o->this.id, info->change_mask);
 
-	info = o->this.info = pw_device_info_update(o->this.info, info);
+	info = o->this.info = pw_device_info_merge(o->this.info, info, o->this.changed == 0);
 
 	if (info->change_mask & PW_DEVICE_CHANGE_MASK_PROPS)
 		changed++;
@@ -285,11 +306,17 @@ static void device_event_info(void *object, const struct pw_device_info *info)
 	if (info->change_mask & PW_DEVICE_CHANGE_MASK_PARAMS) {
 		for (i = 0; i < info->n_params; i++) {
 			uint32_t id = info->params[i].id;
+			int res;
 
 			if (info->params[i].user == 0)
 				continue;
 			info->params[i].user = 0;
 
+			if (id >= MAX_PARAMS) {
+				pw_log_error("too big param id %d", id);
+				continue;
+			}
+
 			switch (id) {
 			case SPA_PARAM_EnumProfile:
 			case SPA_PARAM_Profile:
@@ -299,12 +326,14 @@ static void device_event_info(void *object, const struct pw_device_info *info)
 			case SPA_PARAM_Route:
 				break;
 			}
-			add_param(&o->pending_list, id, NULL);
+			add_param(&o->pending_list, o->param_seq[id], o->param_seq, id, NULL);
 			if (!(info->params[i].flags & SPA_PARAM_INFO_READ))
 				continue;
 
-			pw_device_enum_params((struct pw_device*)o->this.proxy,
-					0, id, 0, -1, NULL);
+			res = pw_device_enum_params((struct pw_device*)o->this.proxy,
+					++o->param_seq[id], id, 0, -1, NULL);
+			if (SPA_RESULT_IS_ASYNC(res))
+				o->param_seq[id] = res;
 		}
 	}
 	if (changed) {
@@ -320,7 +349,7 @@ static struct object *find_device(struct manager *m, uint32_t card_id, uint32_t
 		struct pw_node_info *info;
 		const char *str;
 
-		if (strcmp(o->this.type, PW_TYPE_INTERFACE_Node) != 0)
+		if (!spa_streq(o->this.type, PW_TYPE_INTERFACE_Node))
 			continue;
 
 		if ((info = o->this.info) != NULL &&
@@ -341,13 +370,15 @@ static void device_event_param(void *object, int seq,
 	struct manager *m = o->manager;
 	struct pw_manager_param *p;
 
-	p = add_param(&o->pending_list, id, param);
+	p = add_param(&o->pending_list, seq, o->param_seq, id, param);
+	if (p == NULL)
+		return;
 
 	if (id == SPA_PARAM_Route && !has_param(&o->this.param_list, p)) {
-		uint32_t id, device;
+		uint32_t idx, device;
 		if (spa_pod_parse_object(param,
 				SPA_TYPE_OBJECT_ParamRoute, NULL,
-				SPA_PARAM_ROUTE_index, SPA_POD_Int(&id),
+				SPA_PARAM_ROUTE_index, SPA_POD_Int(&idx),
 				SPA_PARAM_ROUTE_device,  SPA_POD_Int(&device)) < 0)
 			return;
 
@@ -387,7 +418,7 @@ static void node_event_info(void *object, const struct pw_node_info *info)
 
 	pw_log_debug("object %p: id:%d change-mask:%08"PRIx64, o, o->this.id, info->change_mask);
 
-	info = o->this.info = pw_node_info_update(o->this.info, info);
+	info = o->this.info = pw_node_info_merge(o->this.info, info, o->this.changed == 0);
 
 	if (info->change_mask & PW_NODE_CHANGE_MASK_STATE)
 		changed++;
@@ -398,18 +429,26 @@ static void node_event_info(void *object, const struct pw_node_info *info)
 	if (info->change_mask & PW_NODE_CHANGE_MASK_PARAMS) {
 		for (i = 0; i < info->n_params; i++) {
 			uint32_t id = info->params[i].id;
+			int res;
 
 			if (info->params[i].user == 0)
 				continue;
 			info->params[i].user = 0;
 
+			if (id >= MAX_PARAMS) {
+				pw_log_error("too big param id %d", id);
+				continue;
+			}
+
 			changed++;
-			add_param(&o->pending_list, id, NULL);
+			add_param(&o->pending_list, o->param_seq[id], o->param_seq, id, NULL);
 			if (!(info->params[i].flags & SPA_PARAM_INFO_READ))
 				continue;
 
-			pw_node_enum_params((struct pw_node*)o->this.proxy,
-					0, id, 0, -1, NULL);
+			res = pw_node_enum_params((struct pw_node*)o->this.proxy,
+					++o->param_seq[id], id, 0, -1, NULL);
+			if (SPA_RESULT_IS_ASYNC(res))
+				o->param_seq[id] = res;
 		}
 	}
 	if (changed) {
@@ -423,7 +462,7 @@ static void node_event_param(void *object, int seq,
 		const struct spa_pod *param)
 {
 	struct object *o = object;
-	add_param(&o->pending_list, id, param);
+	add_param(&o->pending_list, seq, o->param_seq, id, param);
 }
 
 static const struct pw_node_events node_events = {
@@ -501,7 +540,7 @@ static const struct object_info *find_info(const char *type, uint32_t version)
 {
 	size_t i;
 	for (i = 0; i < SPA_N_ELEMENTS(objects); i++) {
-		if (strcmp(objects[i]->type, type) == 0 &&
+		if (spa_streq(objects[i]->type, type) &&
 		    objects[i]->version <= version)
 			return objects[i];
 	}
@@ -520,11 +559,13 @@ destroy_proxy(void *data)
 {
 	struct object *o = data;
 
+	spa_assert(o->info);
+
 	if (o->info->events)
 		spa_hook_remove(&o->object_listener);
 	spa_hook_remove(&o->proxy_listener);
 
-	if (o->info && o->info->destroy)
+	if (o->info->destroy)
                 o->info->destroy(o);
 
         o->this.proxy = NULL;
@@ -614,7 +655,7 @@ static const struct pw_registry_events registry_events = {
 static void on_core_info(void *data, const struct pw_core_info *info)
 {
 	struct manager *m = data;
-	m->this.info = pw_core_info_update(m->this.info, info);
+	m->this.info = pw_core_info_merge(m->this.info, info, true);
 }
 
 static void on_core_done(void *data, uint32_t id, int seq)
@@ -711,6 +752,8 @@ int pw_manager_set_metadata(struct pw_manager *manager,
 		return -ENOTSUP;
 	if (!SPA_FLAG_IS_SET(metadata->permissions, PW_PERM_W|PW_PERM_X))
 		return -EACCES;
+	if (metadata->proxy == NULL)
+		return -ENOENT;
 
 	if (type != NULL) {
 		va_start(args, format);
@@ -767,7 +810,7 @@ static struct object_data *object_find_data(struct object *o, const char *id)
 {
 	struct object_data *d;
 	spa_list_for_each(d, &o->data_list, link) {
-		if (strcmp(d->id, id) == 0)
+		if (spa_streq(d->id, id))
 			return d;
 	}
 	return NULL;
@@ -793,7 +836,7 @@ void *pw_manager_object_add_data(struct pw_manager_object *obj, const char *id,
 	spa_list_append(&o->data_list, &d->link);
 
 done:
-	return SPA_MEMBER(d, sizeof(struct object_data), void);
+	return SPA_PTROFF(d, sizeof(struct object_data), void);
 }
 
 int pw_manager_sync(struct pw_manager *manager)
@@ -801,3 +844,94 @@ int pw_manager_sync(struct pw_manager *manager)
 	struct manager *m = SPA_CONTAINER_OF(manager, struct manager, this);
 	return core_sync(m);
 }
+
+bool pw_manager_object_is_client(struct pw_manager_object *o)
+{
+	return spa_streq(o->type, PW_TYPE_INTERFACE_Client);
+}
+
+bool pw_manager_object_is_module(struct pw_manager_object *o)
+{
+	return spa_streq(o->type, PW_TYPE_INTERFACE_Module);
+}
+
+bool pw_manager_object_is_card(struct pw_manager_object *o)
+{
+	const char *str;
+	return spa_streq(o->type, PW_TYPE_INTERFACE_Device) &&
+		o->props != NULL &&
+		(str = pw_properties_get(o->props, PW_KEY_MEDIA_CLASS)) != NULL &&
+		spa_streq(str, "Audio/Device");
+}
+
+bool pw_manager_object_is_sink(struct pw_manager_object *o)
+{
+	const char *str;
+	return spa_streq(o->type, PW_TYPE_INTERFACE_Node) &&
+		o->props != NULL &&
+		(str = pw_properties_get(o->props, PW_KEY_MEDIA_CLASS)) != NULL &&
+		(spa_streq(str, "Audio/Sink") || spa_streq(str, "Audio/Duplex"));
+}
+
+bool pw_manager_object_is_source(struct pw_manager_object *o)
+{
+	const char *str;
+	return spa_streq(o->type, PW_TYPE_INTERFACE_Node) &&
+		o->props != NULL &&
+		(str = pw_properties_get(o->props, PW_KEY_MEDIA_CLASS)) != NULL &&
+		(spa_streq(str, "Audio/Source") ||
+		 spa_streq(str, "Audio/Duplex") ||
+		 spa_streq(str, "Audio/Source/Virtual"));
+}
+
+bool pw_manager_object_is_monitor(struct pw_manager_object *o)
+{
+	const char *str;
+	return spa_streq(o->type, PW_TYPE_INTERFACE_Node) &&
+		o->props != NULL &&
+		(str = pw_properties_get(o->props, PW_KEY_MEDIA_CLASS)) != NULL &&
+		(spa_streq(str, "Audio/Sink"));
+}
+
+bool pw_manager_object_is_virtual(struct pw_manager_object *o)
+{
+	const char *str;
+	struct pw_node_info *info;
+	return spa_streq(o->type, PW_TYPE_INTERFACE_Node) &&
+		(info = o->info) != NULL && info->props != NULL &&
+		(str = spa_dict_lookup(info->props, PW_KEY_NODE_VIRTUAL)) != NULL &&
+		pw_properties_parse_bool(str);
+}
+
+bool pw_manager_object_is_source_or_monitor(struct pw_manager_object *o)
+{
+	return pw_manager_object_is_source(o) || pw_manager_object_is_monitor(o);
+}
+
+bool pw_manager_object_is_sink_input(struct pw_manager_object *o)
+{
+	const char *str;
+	return spa_streq(o->type, PW_TYPE_INTERFACE_Node) &&
+		o->props != NULL &&
+		(str = pw_properties_get(o->props, PW_KEY_MEDIA_CLASS)) != NULL &&
+		spa_streq(str, "Stream/Output/Audio");
+}
+
+bool pw_manager_object_is_source_output(struct pw_manager_object *o)
+{
+	const char *str;
+	return spa_streq(o->type, PW_TYPE_INTERFACE_Node) &&
+		o->props != NULL &&
+		(str = pw_properties_get(o->props, PW_KEY_MEDIA_CLASS)) != NULL &&
+		spa_streq(str, "Stream/Input/Audio");
+}
+
+bool pw_manager_object_is_recordable(struct pw_manager_object *o)
+{
+	return pw_manager_object_is_source(o) || pw_manager_object_is_sink(o) || pw_manager_object_is_sink_input(o);
+}
+
+bool pw_manager_object_is_link(struct pw_manager_object *o)
+{
+	return spa_streq(o->type, PW_TYPE_INTERFACE_Link);
+}
