@@ -39,6 +39,7 @@
 #include "config.h"
 
 #include <spa/utils/result.h>
+#include <spa/utils/string.h>
 #include <spa/utils/json.h>
 #include <spa/pod/pod.h>
 #include <spa/pod/builder.h>
@@ -48,6 +49,9 @@
 
 #include <pipewire/impl.h>
 
+/** \page page_module_protocol_simple PipeWire Module: Protocol Simple
+ */
+
 #define NAME "protocol-simple"
 
 #define DEFAULT_PORT 4711
@@ -59,6 +63,8 @@
 #define DEFAULT_POSITION "[ FL FR ]"
 #define DEFAULT_LATENCY "1024/48000"
 
+#define MAX_CLIENTS	10
+
 #define MODULE_USAGE	"[ capture=<bool> ] "						\
 			"[ playback=<bool> ] "						\
 			"[ capture.node=<source-target> ] "				\
@@ -114,6 +120,7 @@ struct client {
 
 	unsigned int disconnect:1;
 	unsigned int disconnecting:1;
+	unsigned int cleanup:1;
 };
 
 struct server {
@@ -128,6 +135,7 @@ struct server {
 	struct spa_source *source;
 
 	struct spa_list client_list;
+	uint32_t n_clients;
 };
 
 static void client_disconnect(struct client *client)
@@ -151,7 +159,10 @@ static void client_free(struct client *client)
 
 	client_disconnect(client);
 
+	pw_work_queue_cancel(impl->work_queue, client, SPA_ID_INVALID);
+
 	spa_list_remove(&client->link);
+	client->server->n_clients--;
 
 	if (client->capture)
 		pw_stream_destroy(client->capture);
@@ -175,7 +186,10 @@ static void on_client_cleanup(void *obj, void *data, int res, uint32_t id)
 static void client_cleanup(struct client *client)
 {
 	struct impl *impl = client->impl;
-	pw_work_queue_add(impl->work_queue, client, 0, on_client_cleanup, impl);
+	if (!client->cleanup)  {
+		client->cleanup = true;
+		pw_work_queue_add(impl->work_queue, client, 0, on_client_cleanup, impl);
+	}
 }
 
 static void
@@ -215,7 +229,7 @@ static void capture_process(void *data)
 	int res;
 
 	if ((buf = pw_stream_dequeue_buffer(client->capture)) == NULL) {
-		pw_log_warn("%p: client:%p [%s] out of capture buffers: %m", impl,
+		pw_log_debug("%p: client:%p [%s] out of capture buffers: %m", impl,
 				client, client->name);
 		return;
 	}
@@ -226,7 +240,7 @@ static void capture_process(void *data)
 
 	while (size > 0) {
 		res = send(client->source->fd,
-				SPA_MEMBER(d->data, offset, void),
+				SPA_PTROFF(d->data, offset, void),
 				size,
 				MSG_NOSIGNAL | MSG_DONTWAIT);
 		if (res < 0) {
@@ -254,7 +268,7 @@ static void playback_process(void *data)
 	int res;
 
 	if ((buf = pw_stream_dequeue_buffer(client->playback)) == NULL) {
-		pw_log_warn("%p: client:%p [%s] out of playback buffers: %m", impl,
+		pw_log_debug("%p: client:%p [%s] out of playback buffers: %m", impl,
 				client, client->name);
 		return;
 	}
@@ -270,7 +284,7 @@ static void playback_process(void *data)
 	offset = 0;
 	while (size > 0) {
 		res = recv(client->source->fd,
-				SPA_MEMBER(d->data, offset, void),
+				SPA_PTROFF(d->data, offset, void),
 				size,
 				MSG_DONTWAIT);
 		if (res == 0) {
@@ -368,7 +382,7 @@ static int create_streams(struct impl *impl, struct client *client)
 
 	if (impl->capture) {
 		props = pw_properties_new(
-			PW_KEY_NODE_GROUP, client->name,
+			PW_KEY_NODE_GROUP, "pipewire.dummy",
 			PW_KEY_NODE_LATENCY, DEFAULT_LATENCY,
 			PW_KEY_NODE_TARGET, pw_properties_get(impl->props, "capture.node"),
 			NULL);
@@ -388,7 +402,7 @@ static int create_streams(struct impl *impl, struct client *client)
 	}
 	if (impl->playback) {
 		props = pw_properties_new(
-			PW_KEY_NODE_GROUP, client->name,
+			PW_KEY_NODE_GROUP, "pipewire.dummy",
 			PW_KEY_NODE_LATENCY, DEFAULT_LATENCY,
 			PW_KEY_NODE_TARGET, pw_properties_get(impl->props, "playback.node"),
 			NULL);
@@ -443,7 +457,7 @@ static void on_core_proxy_destroy(void *data)
 	client_cleanup(client);
 }
 
-static struct pw_proxy_events core_proxy_events = {
+static const struct pw_proxy_events core_proxy_events = {
 	PW_VERSION_CORE_EVENTS,
 	.destroy = on_core_proxy_destroy,
 };
@@ -464,6 +478,12 @@ on_connect(void *data, int fd, uint32_t mask)
 	if (client_fd < 0)
 		goto error;
 
+	if (server->n_clients >= MAX_CLIENTS) {
+		close(client_fd);
+		errno = ECONNREFUSED;
+		goto error;
+	}
+
 	client = calloc(1, sizeof(struct client));
 	if (client == NULL)
 		goto error;
@@ -471,10 +491,18 @@ on_connect(void *data, int fd, uint32_t mask)
 	client->impl = impl;
 	client->server = server;
 	spa_list_append(&server->client_list, &client->link);
+	server->n_clients++;
 
 	if (inet_ntop(addr.sa_family, addr.sa_data, client->name, sizeof(client->name)) == NULL)
 		snprintf(client->name, sizeof(client->name), "client %d", client_fd);
 
+	client->source = pw_loop_add_io(impl->loop,
+					client_fd,
+					SPA_IO_ERR | SPA_IO_HUP,
+					true, on_client_data, client);
+	if (client->source == NULL)
+		goto error;
+
 	pw_log_info(NAME" %p: client:%p [%s] connected", impl, client, client->name);
 
 	props = pw_properties_new(
@@ -503,13 +531,6 @@ on_connect(void *data, int fd, uint32_t mask)
 		pw_properties_set(props, PW_KEY_CLIENT_ACCESS, "restricted");
 	}
 
-	client->source = pw_loop_add_io(impl->loop,
-					client_fd,
-					SPA_IO_ERR | SPA_IO_HUP,
-					true, on_client_data, client);
-	if (client->source == NULL)
-		goto error;
-
 	client->core = pw_context_connect(impl->context, props, 0);
 	props = NULL;
 	if (client->core == NULL)
@@ -524,8 +545,7 @@ on_connect(void *data, int fd, uint32_t mask)
 	return;
 error:
 	pw_log_error(NAME" %p: failed to create client: %m", impl);
-	if (props != NULL)
-		pw_properties_free(props);
+	pw_properties_free(props);
 	if (client != NULL)
 		client_free(client);
 	return;
@@ -620,7 +640,7 @@ static struct server *create_server(struct impl *impl, const char *address)
 	spa_list_init(&server->client_list);
 	spa_list_append(&impl->server_list, &server->link);
 
-	if (strstr(address, "tcp:") == address) {
+	if (spa_strstartswith(address, "tcp:")) {
 		fd = make_inet_socket(server, address+4);
 	} else {
 		fd = -EINVAL;
@@ -652,8 +672,7 @@ static void impl_free(struct impl *impl)
 	spa_hook_remove(&impl->module_listener);
 	spa_list_consume(s, &impl->server_list, link)
 		server_free(s);
-	if (impl->props)
-		pw_properties_free(impl->props);
+	pw_properties_free(impl->props);
 	free(impl);
 }
 
@@ -671,7 +690,7 @@ static inline uint32_t channel_from_name(const char *name)
 {
 	int i;
 	for (i = 0; spa_type_audio_channel[i].name; i++) {
-		if (strcmp(name, spa_debug_type_short_name(spa_type_audio_channel[i].name)) == 0)
+		if (spa_streq(name, spa_debug_type_short_name(spa_type_audio_channel[i].name)))
 			return spa_type_audio_channel[i].type;
 	}
 	return SPA_AUDIO_CHANNEL_UNKNOWN;
