@@ -117,6 +117,8 @@ struct port {
 	struct queue dequeued;
 	struct queue queued;
 
+	struct spa_latency_info latency[2];
+
 	/* from here is what the caller gets as user_data */
 	uint8_t user_data[0];
 };
@@ -145,10 +147,13 @@ struct filter {
 	uint32_t change_mask_all;
 	struct spa_node_info info;
 	struct spa_list param_list;
-#define IDX_Props	0
-#define N_NODE_PARAMS	1
+#define IDX_Props		0
+#define IDX_ProcessLatency	1
+#define N_NODE_PARAMS		2
 	struct spa_param_info params[N_NODE_PARAMS];
 
+	struct spa_process_latency_info process_latency;
+
 	struct data data;
 	uintptr_t seq;
 	struct pw_time time;
@@ -171,6 +176,8 @@ static int get_param_index(uint32_t id)
 	switch (id) {
 	case SPA_PARAM_Props:
 		return IDX_Props;
+	case SPA_PARAM_ProcessLatency:
+		return IDX_ProcessLatency;
 	default:
 		return -1;
 	}
@@ -196,6 +203,32 @@ static int get_port_param_index(uint32_t id)
 	}
 }
 
+static void fix_datatype(const struct spa_pod *param)
+{
+	const struct spa_pod_prop *pod_param;
+	const struct spa_pod *vals;
+	uint32_t dataType, n_vals, choice;
+
+	pod_param = spa_pod_find_prop(param, NULL, SPA_PARAM_BUFFERS_dataType);
+	if (pod_param == NULL)
+		return;
+
+	vals = spa_pod_get_values(&pod_param->value, &n_vals, &choice);
+	if (n_vals == 0)
+		return;
+
+	if (spa_pod_get_int(&vals[0], (int32_t*)&dataType) < 0)
+		return;
+
+	pw_log_debug(NAME" dataType: %u", dataType);
+	if (dataType & (1u << SPA_DATA_MemPtr)) {
+		SPA_POD_VALUE(struct spa_pod_int, &vals[0]) =
+			dataType | mappable_dataTypes;
+		pw_log_debug(NAME" Change dataType: %u -> %u", dataType,
+				SPA_POD_VALUE(struct spa_pod_int, &vals[0]));
+	}
+}
+
 static struct param *add_param(struct filter *impl, struct port *port,
 		uint32_t id, uint32_t flags, const struct spa_pod *param)
 {
@@ -213,27 +246,13 @@ static struct param *add_param(struct filter *impl, struct port *port,
 	if (p == NULL)
 		return NULL;
 
-	if (id == SPA_PARAM_Buffers && SPA_FLAG_IS_SET(port->flags, PW_FILTER_PORT_FLAG_MAP_BUFFERS) &&
-		port->direction == SPA_DIRECTION_INPUT)
-	{
-		const struct spa_pod_prop *pod_param;
-		uint32_t dataType = 0;
+	if (id == SPA_PARAM_Buffers && port != NULL &&
+	    SPA_FLAG_IS_SET(port->flags, PW_FILTER_PORT_FLAG_MAP_BUFFERS) &&
+	    port->direction == SPA_DIRECTION_INPUT)
+		fix_datatype(param);
 
-		pod_param = spa_pod_find_prop(param, NULL, SPA_PARAM_BUFFERS_dataType);
-		if (pod_param != NULL)
-		{
-			spa_pod_get_int(&pod_param->value, (int32_t*)&dataType);
-			pw_log_debug(NAME" dataType: %d", dataType);
-			if ((dataType & (1<<SPA_DATA_MemPtr)) > 0)
-			{
-				pw_log_debug(NAME" Change dataType");
-				struct spa_pod_int *int_pod = (struct spa_pod_int*)&pod_param->value;
-				dataType = dataType | mappable_dataTypes;
-				pw_log_debug(NAME" dataType: %d", dataType);
-				int_pod->value = dataType;
-			}
-		}
-	}
+	if (id == SPA_PARAM_ProcessLatency && port == NULL)
+		spa_process_latency_parse(param, &impl->process_latency);
 
 	p->id = id;
 	p->flags = flags;
@@ -241,13 +260,16 @@ static struct param *add_param(struct filter *impl, struct port *port,
 	memcpy(p->param, param, SPA_POD_SIZE(param));
 	SPA_POD_OBJECT_ID(p->param) = id;
 
+	pw_log_debug(NAME" %p: port %p param id %d (%s)", impl, p, id,
+			spa_debug_type_find_name(spa_type_param, id));
+
 	if (port) {
 		idx = get_port_param_index(id);
 		spa_list_append(&port->param_list, &p->link);
 		if (idx != -1) {
 			port->info.change_mask |= SPA_PORT_CHANGE_MASK_PARAMS;
 			port->params[idx].flags |= SPA_PARAM_INFO_READ;
-			port->params[idx].flags ^= SPA_PARAM_INFO_SERIAL;
+			port->params[idx].user++;
 		}
 	} else {
 		idx = get_param_index(id);
@@ -255,7 +277,7 @@ static struct param *add_param(struct filter *impl, struct port *port,
 		if (idx != -1) {
 			impl->info.change_mask |= SPA_NODE_CHANGE_MASK_PARAMS;
 			impl->params[idx].flags |= SPA_PARAM_INFO_READ;
-			impl->params[idx].flags ^= SPA_PARAM_INFO_SERIAL;
+			impl->params[idx].user++;
 		}
 	}
 	return p;
@@ -297,6 +319,8 @@ static struct port *alloc_port(struct filter *filter,
 	p->filter = filter;
 	p->direction = direction;
 	p->id = i;
+	p->latency[SPA_DIRECTION_INPUT] = SPA_LATENCY_INFO(SPA_DIRECTION_INPUT);
+	p->latency[SPA_DIRECTION_OUTPUT] = SPA_LATENCY_INFO(SPA_DIRECTION_OUTPUT);
 
 	spa_list_init(&p->param_list);
 	spa_ringbuffer_init(&p->dequeued.ring);
@@ -310,6 +334,9 @@ static struct port *alloc_port(struct filter *filter,
 
 static inline struct port *get_port(struct filter *filter, enum spa_direction direction, uint32_t port_id)
 {
+	if ((direction != SPA_DIRECTION_INPUT && direction != SPA_DIRECTION_OUTPUT) ||
+	    port_id >= MAX_PORTS)
+		return NULL;
 	return filter->ports[direction][port_id];
 }
 
@@ -378,6 +405,69 @@ static bool filter_set_state(struct pw_filter *filter, enum pw_filter_state stat
 	return res;
 }
 
+static int enum_params(struct filter *d, struct spa_list *param_list, int seq,
+		uint32_t id, uint32_t start, uint32_t num, const struct spa_pod *filter)
+{
+	struct spa_result_node_params result;
+	uint8_t buffer[1024];
+	struct spa_pod_builder b = { 0 };
+	uint32_t count = 0;
+	struct param *p;
+	bool found = false;
+
+	spa_return_val_if_fail(num != 0, -EINVAL);
+
+	result.id = id;
+	result.next = 0;
+
+	pw_log_debug(NAME" %p: %p param id %d (%s) start:%d num:%d", d, param_list, id,
+			spa_debug_type_find_name(spa_type_param, id),
+			start, num);
+
+	spa_list_for_each(p, param_list, link) {
+		struct spa_pod *param;
+
+		result.index = result.next++;
+		if (result.index < start)
+			continue;
+
+		param = p->param;
+		if (param == NULL || p->id != id)
+			continue;
+
+		found = true;
+
+		spa_pod_builder_init(&b, buffer, sizeof(buffer));
+		if (spa_pod_filter(&b, &result.param, param, filter) != 0)
+			continue;
+
+		spa_node_emit_result(&d->hooks, seq, 0, SPA_RESULT_TYPE_NODE_PARAMS, &result);
+
+		if (++count == num)
+			break;
+	}
+	return found ? 0 : -ENOENT;
+}
+
+static int impl_enum_params(void *object, int seq, uint32_t id, uint32_t start, uint32_t num,
+				 const struct spa_pod *filter)
+{
+	struct filter *impl = object;
+	return enum_params(impl, &impl->param_list, seq, id, start, num, filter);
+}
+
+static int impl_set_param(void *object, uint32_t id, uint32_t flags, const struct spa_pod *param)
+{
+	struct filter *impl = object;
+	struct pw_filter *filter = &impl->this;
+
+	if (id != SPA_PARAM_Props)
+		return -ENOTSUP;
+
+	pw_filter_emit_param_changed(filter, NULL, id, param);
+	return 0;
+}
+
 static int
 do_set_position(struct spa_loop *loop,
 		bool async, uint32_t seq, const void *data, size_t size, void *user_data)
@@ -439,21 +529,41 @@ static int impl_send_command(void *object, const struct spa_command *command)
 
 static void emit_node_info(struct filter *d, bool full)
 {
+	uint32_t i;
 	uint64_t old = full ? d->info.change_mask : 0;
 	if (full)
 		d->info.change_mask = d->change_mask_all;
-	if (d->info.change_mask != 0)
+	if (d->info.change_mask != 0) {
+		if (d->info.change_mask & SPA_NODE_CHANGE_MASK_PARAMS) {
+			for (i = 0; i < d->info.n_params; i++) {
+				if (d->params[i].user > 0) {
+					d->params[i].flags ^= SPA_PARAM_INFO_SERIAL;
+					d->params[i].user = 0;
+				}
+			}
+		}
 		spa_node_emit_info(&d->hooks, &d->info);
+	}
 	d->info.change_mask = old;
 }
 
 static void emit_port_info(struct filter *d, struct port *p, bool full)
 {
+	uint32_t i;
 	uint64_t old = full ? p->info.change_mask : 0;
 	if (full)
 		p->info.change_mask = p->change_mask_all;
-	if (p->info.change_mask != 0)
+	if (p->info.change_mask != 0) {
+		if (p->info.change_mask & SPA_PORT_CHANGE_MASK_PARAMS) {
+			for (i = 0; i < p->info.n_params; i++) {
+				if (p->params[i].user > 0) {
+					p->params[i].flags ^= SPA_PARAM_INFO_SERIAL;
+					p->params[i].user = 0;
+				}
+			}
+		}
 		spa_node_emit_port_info(&d->hooks, p->direction, p->id, &p->info);
+	}
 	p->info.change_mask = old;
 }
 
@@ -521,48 +631,11 @@ static int impl_port_enum_params(void *object, int seq,
 {
 	struct filter *d = object;
 	struct port *port;
-	struct spa_result_node_params result;
-	uint8_t buffer[1024];
-	struct spa_pod_builder b = { 0 };
-	uint32_t count = 0;
-	struct param *p;
-	bool found = false;
-
-	spa_return_val_if_fail(num != 0, -EINVAL);
-
-	result.id = id;
-	result.next = 0;
 
 	if ((port = get_port(d, direction, port_id)) == NULL)
 		return -EINVAL;
 
-	pw_log_debug(NAME" %p: port %p param id %d (%s) start:%d num:%d", d, port, id,
-			spa_debug_type_find_name(spa_type_param, id),
-			start, num);
-
-	spa_list_for_each(p, &port->param_list, link) {
-		struct spa_pod *param;
-
-		result.index = result.next++;
-		if (result.index < start)
-			continue;
-
-		param = p->param;
-		if (param == NULL || p->id != id)
-			continue;
-
-		found = true;
-
-		spa_pod_builder_init(&b, buffer, sizeof(buffer));
-		if (spa_pod_filter(&b, &result.param, param, filter) != 0)
-			continue;
-
-		spa_node_emit_result(&d->hooks, seq, 0, SPA_RESULT_TYPE_NODE_PARAMS, &result);
-
-		if (++count == num)
-			break;
-	}
-	return found ? 0 : -ENOENT;
+	return enum_params(d, &port->param_list, seq, id, start, num, filter);
 }
 
 static int update_params(struct filter *impl, struct port *port, uint32_t id,
@@ -570,6 +643,7 @@ static int update_params(struct filter *impl, struct port *port, uint32_t id,
 {
 	uint32_t i;
 	int res = 0;
+	bool update_latency = false;
 
 	if (id != SPA_ID_INVALID) {
 		clear_params(impl, port, id);
@@ -581,11 +655,39 @@ static int update_params(struct filter *impl, struct port *port, uint32_t id,
 		}
 	}
 	for (i = 0; i < n_params; i++) {
+		if (params[i] == NULL)
+			continue;
+
+		if (port != NULL &&
+		    spa_pod_is_object(params[i]) &&
+		    SPA_POD_OBJECT_ID(params[i]) == SPA_PARAM_Latency) {
+			struct spa_latency_info info;
+			if (spa_latency_parse(params[i], &info) >= 0) {
+				port->latency[info.direction] = info;
+				pw_log_debug("port %p: set %s latency %f-%f %d-%d %"PRIu64"-%"PRIu64, port,
+					info.direction == SPA_DIRECTION_INPUT ? "input" : "output",
+					info.min_quantum, info.max_quantum,
+					info.min_rate, info.max_rate,
+					info.min_ns, info.max_ns);
+				update_latency = true;
+			}
+			continue;
+		}
 		if (add_param(impl, port, id, 0, params[i]) == NULL) {
 			res = -errno;
 			break;
 		}
 	}
+	if (port != NULL && update_latency) {
+		uint8_t buffer[4096];
+		struct spa_pod_builder b;
+
+		spa_pod_builder_init(&b, buffer, sizeof(buffer));
+		add_param(impl, port, SPA_PARAM_Latency, 0,
+				spa_latency_build(&b, SPA_PARAM_Latency, &port->latency[0]));
+		add_param(impl, port, SPA_PARAM_Latency, 0,
+				spa_latency_build(&b, SPA_PARAM_Latency, &port->latency[1]));
+	}
 	return res;
 }
 
@@ -659,6 +761,68 @@ static void clear_buffers(struct port *port)
 	clear_queue(port, &port->queued);
 }
 
+
+static int default_latency(struct filter *impl, struct port *port, enum spa_direction direction)
+{
+	struct pw_filter *filter = &impl->this;
+	struct spa_latency_info info;
+	struct port *p;
+
+	info = SPA_LATENCY_INFO(direction);
+
+	spa_list_for_each(p, &impl->port_list, link) {
+		if (p->direction == direction)
+			continue;
+		spa_latency_info_combine(&info, &p->latency[direction]);
+	}
+
+	spa_process_latency_info_add(&impl->process_latency, &info);
+
+	spa_list_for_each(p, &impl->port_list, link) {
+		uint8_t buffer[4096];
+		struct spa_pod_builder b;
+		const struct spa_pod *params[1];
+
+		if (p->direction != direction)
+			continue;
+
+		spa_pod_builder_init(&b, buffer, sizeof(buffer));
+		params[0] = spa_latency_build(&b, SPA_PARAM_Latency, &info);
+		pw_filter_update_params(filter, p->user_data, params, 1);
+	}
+	return 0;
+}
+
+static int handle_latency(struct filter *impl, struct port *port, const struct spa_pod *param)
+{
+	struct pw_filter *filter = &impl->this;
+	struct spa_latency_info info;
+	int res;
+
+	if (param == NULL)
+		return 0;
+
+	if ((res = spa_latency_parse(param, &info)) < 0)
+		return res;
+
+	pw_log_info("port %p: set %s latency %f-%f %d-%d %"PRIu64"-%"PRIu64, port,
+			info.direction == SPA_DIRECTION_INPUT ? "input" : "output",
+			info.min_quantum, info.max_quantum,
+			info.min_rate, info.max_rate,
+			info.min_ns, info.max_ns);
+
+	if (info.direction == port->direction)
+		return 0;
+
+	if (SPA_FLAG_IS_SET(impl->flags, PW_FILTER_FLAG_CUSTOM_LATENCY)) {
+		pw_filter_emit_param_changed(filter, port->user_data,
+				SPA_PARAM_Latency, param);
+	} else {
+		default_latency(impl, port, info.direction);
+	}
+	return 0;
+}
+
 static int impl_port_set_param(void *object,
 			       enum spa_direction direction, uint32_t port_id,
 			       uint32_t id, uint32_t flags,
@@ -668,6 +832,9 @@ static int impl_port_set_param(void *object,
 	struct pw_filter *filter = &impl->this;
 	struct port *port;
 	int res;
+	bool emit = true;
+	const struct spa_pod *params[1];
+	uint32_t n_params = 0;
 
 	if (impl->disconnecting && param != NULL)
 		return -EIO;
@@ -680,13 +847,24 @@ static int impl_port_set_param(void *object,
 	if (param)
 		pw_log_pod(SPA_LOG_LEVEL_DEBUG, param);
 
-	if ((res = update_params(impl, port, id, &param, param ? 1 : 0)) < 0)
+	params[0] = param;
+	n_params = param ? 1 : 0;
+
+	if ((res = update_params(impl, port, id, params, n_params)) < 0)
 		return res;
 
-	if (id == SPA_PARAM_Format)
+	switch (id) {
+	case SPA_PARAM_Format:
 		clear_buffers(port);
+		break;
+	case SPA_PARAM_Latency:
+		handle_latency(impl, port, param);
+		emit = false;
+		break;
+	}
 
-	pw_filter_emit_param_changed(filter, port->user_data, id, param);
+	if (emit)
+		pw_filter_emit_param_changed(filter, port->user_data, id, param);
 
 	if (filter->state == PW_FILTER_STATE_ERROR)
 		return -EIO;
@@ -928,6 +1106,8 @@ static const struct spa_node_methods impl_node = {
 	SPA_VERSION_NODE_METHODS,
 	.add_listener = impl_add_listener,
 	.set_callbacks = impl_set_callbacks,
+	.enum_params = impl_enum_params,
+	.set_param = impl_set_param,
 	.set_io = impl_set_io,
 	.send_command = impl_send_command,
 	.port_set_io = impl_port_set_io,
@@ -1048,8 +1228,8 @@ filter_new(struct pw_context *context, const char *name,
 	this->state = PW_FILTER_STATE_UNCONNECTED;
 
 	impl->context = context;
-	impl->allow_mlock = context->defaults.mem_allow_mlock;
-	impl->warn_mlock = context->defaults.mem_warn_mlock;
+	impl->allow_mlock = context->settings.mem_allow_mlock;
+	impl->warn_mlock = context->settings.mem_warn_mlock;
 
 	return impl;
 
@@ -1277,6 +1457,7 @@ pw_filter_connect(struct pw_filter *filter,
 	const char *str;
 	int res;
 	uint32_t i;
+	struct spa_dict_item items[1];
 
 	pw_log_debug(NAME" %p: connect", filter);
 	impl->flags = flags;
@@ -1302,6 +1483,7 @@ pw_filter_connect(struct pw_filter *filter,
 	impl->info.flags = impl->process_rt ? SPA_NODE_FLAG_RT : 0;
 	impl->info.props = &filter->properties->dict;
 	impl->params[IDX_Props] = SPA_PARAM_INFO(SPA_PARAM_Props, SPA_PARAM_INFO_WRITE);
+	impl->params[IDX_ProcessLatency] = SPA_PARAM_INFO(SPA_PARAM_ProcessLatency, 0);
 	impl->info.params = impl->params;
 	impl->info.n_params = N_NODE_PARAMS;
 	impl->info.change_mask = impl->change_mask_all;
@@ -1328,8 +1510,11 @@ pw_filter_connect(struct pw_filter *filter,
 	}
 
 	pw_log_debug(NAME" %p: export node %p", filter, &impl->impl_node);
+
+	items[0] = SPA_DICT_ITEM_INIT(PW_KEY_OBJECT_REGISTER, "false");
 	filter->proxy = pw_core_export(filter->core,
-			SPA_TYPE_INTERFACE_Node, NULL, &impl->impl_node, 0);
+			SPA_TYPE_INTERFACE_Node, &SPA_DICT_INIT_ARRAY(items),
+			&impl->impl_node, 0);
 	if (filter->proxy == NULL) {
 		res = -errno;
 		goto error_proxy;
@@ -1466,6 +1651,23 @@ void *pw_filter_add_port(struct pw_filter *filter,
 	p->props = props;
 	p->flags = flags;
 
+	p->change_mask_all = SPA_PORT_CHANGE_MASK_FLAGS |
+		SPA_PORT_CHANGE_MASK_PROPS;
+	p->info = SPA_PORT_INFO_INIT();
+	p->info.flags = 0;
+	if (SPA_FLAG_IS_SET(flags, PW_FILTER_PORT_FLAG_ALLOC_BUFFERS))
+		p->info.flags |= SPA_PORT_FLAG_CAN_ALLOC_BUFFERS;
+	p->info.props = &p->props->dict;
+	p->change_mask_all |= SPA_PORT_CHANGE_MASK_PARAMS;
+	p->params[IDX_EnumFormat] = SPA_PARAM_INFO(SPA_PARAM_EnumFormat, 0);
+	p->params[IDX_Meta] = SPA_PARAM_INFO(SPA_PARAM_Meta, 0);
+	p->params[IDX_IO] = SPA_PARAM_INFO(SPA_PARAM_IO, 0);
+	p->params[IDX_Format] = SPA_PARAM_INFO(SPA_PARAM_Format, SPA_PARAM_INFO_WRITE);
+	p->params[IDX_Buffers] = SPA_PARAM_INFO(SPA_PARAM_Buffers, 0);
+	p->params[IDX_Latency] = SPA_PARAM_INFO(SPA_PARAM_Latency, SPA_PARAM_INFO_WRITE);
+	p->info.params = p->params;
+	p->info.n_params = N_PORT_PARAMS;
+
 	/* first configure default params */
 	add_port_params(impl, p);
 	if ((str = pw_properties_get(props, PW_KEY_FORMAT_DSP)) != NULL) {
@@ -1481,23 +1683,6 @@ void *pw_filter_add_port(struct pw_filter *filter,
 	if (update_params(impl, p, SPA_ID_INVALID, params, n_params) < 0)
 		goto error_free;
 
-	p->change_mask_all = SPA_PORT_CHANGE_MASK_FLAGS |
-		SPA_PORT_CHANGE_MASK_PROPS;
-	p->info = SPA_PORT_INFO_INIT();
-	p->info.flags = 0;
-	if (SPA_FLAG_IS_SET(flags, PW_FILTER_PORT_FLAG_ALLOC_BUFFERS))
-		p->info.flags |= SPA_PORT_FLAG_CAN_ALLOC_BUFFERS;
-	p->info.props = &p->props->dict;
-	p->change_mask_all |= SPA_PORT_CHANGE_MASK_PARAMS;
-	p->params[IDX_EnumFormat] = SPA_PARAM_INFO(SPA_PARAM_EnumFormat, SPA_PARAM_INFO_READ);
-	p->params[IDX_Meta] = SPA_PARAM_INFO(SPA_PARAM_Meta, 0);
-	p->params[IDX_IO] = SPA_PARAM_INFO(SPA_PARAM_IO, 0);
-	p->params[IDX_Format] = SPA_PARAM_INFO(SPA_PARAM_Format, SPA_PARAM_INFO_WRITE);
-	p->params[IDX_Buffers] = SPA_PARAM_INFO(SPA_PARAM_Buffers, 0);
-	p->params[IDX_Latency] = SPA_PARAM_INFO(SPA_PARAM_Buffers, SPA_PARAM_INFO_WRITE);
-	p->info.params = p->params;
-	p->info.n_params = N_PORT_PARAMS;
-
 	emit_port_info(impl, p, true);
 
 	return p->user_data;
