@@ -25,6 +25,8 @@
 #include <errno.h>
 #include <string.h>
 
+#include <spa/utils/string.h>
+
 #include "pipewire/impl.h"
 #include "pipewire/private.h"
 
@@ -133,11 +135,11 @@ static int client_error(void *object, uint32_t id, int res, const char *error)
 	return 0;
 }
 
-static bool has_key(const char *keys[], const char *key)
+static bool has_key(const char * const keys[], const char *key)
 {
 	int i;
 	for (i = 0; keys[i]; i++) {
-		if (strcmp(keys[i], key) == 0)
+		if (spa_streq(keys[i], key))
 			return true;
 	}
 	return false;
@@ -145,19 +147,21 @@ static bool has_key(const char *keys[], const char *key)
 
 static int update_properties(struct pw_impl_client *client, const struct spa_dict *dict, bool filter)
 {
-	struct pw_resource *resource;
-	int changed = 0;
-	uint32_t i;
-	const char *old, *ignored[] = {
+	static const char * const ignored[] = {
 		PW_KEY_OBJECT_ID,
 		NULL
 	};
 
+	struct pw_resource *resource;
+	int changed = 0;
+	uint32_t i;
+	const char *old;
+
         for (i = 0; i < dict->n_items; i++) {
 		if (filter) {
-			if (strstr(dict->items[i].key, "pipewire.") == dict->items[i].key &&
+			if (spa_strstartswith(dict->items[i].key, "pipewire.") &&
 			    (old = pw_properties_get(client->properties, dict->items[i].key)) != NULL &&
-			    (dict->items[i].value == NULL || strcmp(old, dict->items[i].value) != 0)) {
+			    (dict->items[i].value == NULL || !spa_streq(old, dict->items[i].value))) {
 				pw_log_warn(NAME" %p: refuse property update '%s' from '%s' to '%s'",
 						client, dict->items[i].key, old,
 						dict->items[i].value);
@@ -198,14 +202,16 @@ static void update_busy(struct pw_impl_client *client)
 
 static int finish_register(struct pw_impl_client *client)
 {
-	struct impl *impl = SPA_CONTAINER_OF(client, struct impl, this);
-	struct pw_impl_client *current;
-	const char *keys[] = {
+	static const char * const keys[] = {
 		PW_KEY_ACCESS,
 		PW_KEY_CLIENT_ACCESS,
 		PW_KEY_APP_NAME,
 		NULL
 	};
+
+	struct impl *impl = SPA_CONTAINER_OF(client, struct impl, this);
+	struct pw_impl_client *current;
+
 	if (impl->registered)
 		return 0;
 
@@ -371,12 +377,10 @@ static const struct pw_context_events context_events = {
 
 /** Make a new client object
  *
- * \param context a \ref pw_context object to register the client with
- * \param ucred a ucred structure or NULL when unknown
+ * \param core a \ref pw_context object to register the client with
  * \param properties optional client properties, ownership is taken
  * \return a newly allocated client object
  *
- * \memberof pw_impl_client
  */
 SPA_EXPORT
 struct pw_impl_client *pw_context_create_client(struct pw_impl_core *core,
@@ -430,7 +434,7 @@ struct pw_impl_client *pw_context_create_client(struct pw_impl_core *core,
 	this->permission_data = impl;
 
 	if (user_data_size > 0)
-		this->user_data = SPA_MEMBER(impl, sizeof(struct impl), void);
+		this->user_data = SPA_PTROFF(impl, sizeof(struct impl), void);
 
 	spa_hook_list_init(&this->listener_list);
 
@@ -447,8 +451,7 @@ error_clear_array:
 error_free:
 	free(impl);
 error_cleanup:
-	if (properties)
-		pw_properties_free(properties);
+	pw_properties_free(properties);
 	errno = -res;
 	return NULL;
 }
@@ -470,8 +473,7 @@ SPA_EXPORT
 int pw_impl_client_register(struct pw_impl_client *client,
 		       struct pw_properties *properties)
 {
-	struct pw_context *context = client->context;
-	const char *keys[] = {
+	static const char * const keys[] = {
 		PW_KEY_MODULE_ID,
 		PW_KEY_PROTOCOL,
 		PW_KEY_SEC_PID,
@@ -481,6 +483,8 @@ int pw_impl_client_register(struct pw_impl_client *client,
 		NULL
 	};
 
+	struct pw_context *context = client->context;
+
 	if (client->registered)
 		goto error_existed;
 
@@ -510,8 +514,7 @@ int pw_impl_client_register(struct pw_impl_client *client,
 	return 0;
 
 error_existed:
-	if (properties)
-		pw_properties_free(properties);
+	pw_properties_free(properties);
 	return -EEXIST;
 }
 
@@ -569,7 +572,6 @@ static int destroy_resource(void *object, void *data)
  *
  * \param client the client to destroy
  *
- * \memberof pw_impl_client
  */
 SPA_EXPORT
 void pw_impl_client_destroy(struct pw_impl_client *client)
@@ -625,13 +627,12 @@ const struct pw_client_info *pw_impl_client_get_info(struct pw_impl_client *clie
 /** Update client properties
  *
  * \param client the client
- * \param dict a \ref spa_dict with properties
+ * \param dict a struct spa_dict with properties
  *
  * Add all properties in \a dict to the client properties. Existing
  * properties are overwritten. Items can be removed by setting the value
  * to NULL.
  *
- * \memberof pw_impl_client
  */
 SPA_EXPORT
 int pw_impl_client_update_properties(struct pw_impl_client *client, const struct spa_dict *dict)
@@ -690,8 +691,10 @@ int pw_impl_client_update_permissions(struct pw_impl_client *client,
 			p = ensure_permissions(client, permissions[i].id);
 			if (p == NULL) {
 				pw_log_warn(NAME" %p: can't ensure permission: %m", client);
-				continue;
+				return -errno;
 			}
+			if ((def = find_permission(client, PW_ID_ANY)) == NULL)
+				return -EIO;
 			old_perm = p->permissions == PW_PERM_INVALID ? def->permissions : p->permissions;
 			new_perm = permissions[i].permissions;
 
