@@ -67,7 +67,7 @@ ssize_t getrandom(void *buf, size_t buflen, unsigned int flags) {
 #include "pipewire/impl.h"
 #include "pipewire/private.h"
 
-#include "extensions/protocol-native.h"
+#include "pipewire/extensions/protocol-native.h"
 
 #define NAME "impl-core"
 
@@ -609,9 +609,7 @@ SPA_EXPORT
 int pw_impl_core_register(struct pw_impl_core *core,
 			 struct pw_properties *properties)
 {
-	struct pw_context *context = core->context;
-	int res;
-	const char *keys[] = {
+	static const char * const keys[] = {
 		PW_KEY_USER_NAME,
 		PW_KEY_HOST_NAME,
 		PW_KEY_CORE_NAME,
@@ -619,6 +617,9 @@ int pw_impl_core_register(struct pw_impl_core *core,
 		NULL
 	};
 
+	struct pw_context *context = core->context;
+	int res;
+
 	if (core->registered)
 		goto error_existed;
 
